# -*- coding: utf-8 -*-

citation = {'ref': u"Anders Krogh, Björn Larsson, Gunnar von Heijne and Erik "
                   u"L. L. Sonnhammer (2001) Predicting Transmembrane Protein "
                   u"Topology with a Hidden Markov Model: Application to "
                   u"Complete Genomes. J. Mol. Biol. 305:567-580. \n"
                   u"<http://dx.doi.org/10.1006/jmbi.2000.4315>",
            'name': "TMHMM 2.0"
           }
          
__DEBUG__ = False

import sys, os, time, json, re
from suds.client import Client
from suds.bindings import binding 
import logging
from inmembrane.helpers import log_stderr, print_proteins

def annotate(params, proteins, \
             #url = 'http://www.cbs.dtu.dk/ws/TMHMM/TMHMM_2_0_ws0.wsdl', \
             #url = 'http://www.cbs.dtu.dk/ws/TMHMM/TMHMM_2_0_ws1.wsdl', \
             url = "http://raw.github.com/boscoh/inmembrane/master/inmembrane/plugins/extra/TMHMM_2_0_ws0.wsdl", \
             batchsize = 2000, \
             force=False):
  mapping = {'TMhelix':'tmhmm_helices',\
              'outside':'tmhmm_outer_loops',\
               'inside':'tmhmm_inner_loops',\
  }
  
  if __DEBUG__:
    logging.basicConfig(level=logging.INFO)
    # soap messages (in&out) and http headers
    logging.getLogger('suds.client').setLevel(logging.DEBUG)             
      
  # grab the cached results if present
  outfile = "tmhmm_web.out"
  if not force and os.path.isfile(outfile):
    log_stderr("# -> skipped: %s already exists" % outfile)
    fh = open(outfile, 'r')
    annots = json.loads(fh.read())
    fh.close()
    for seqid in annots:
      for k in mapping.values():
        proteins[seqid][k] = annots[seqid][k]

    citation['name'] = annots[seqid]['program_name']
    return proteins
  
  log_stderr("# TMHMM(web), %s > %s" % (params['fasta'], outfile))
  log_stderr("# TMHMM(web): submitting in batches of %i sequences" % batchsize)
  
  seqids = proteins.keys()
  tmhmm_dict = {}
  while seqids:
    seqid_batch = seqids[0:batchsize]
    del seqids[0:batchsize]
    client = Client(url, cache=None)
    request=client.factory.create('runService.parameters')
    
    # this is a horrible horrible workaround to account for the fact that
    # the lipop SOAP service returns null results if there is are certain
    # non-alphanumeric characters in the sequence id provided. horrible.
    tmhmm_seq_id_mapping = {}
    seqcount = 0
    
    sys.stderr.write("# ")
    for seqid in seqid_batch:
      seq = client.factory.create('runService.parameters.sequencedata.sequence')

      # workaround: removes any non-alphanumeric character (except '_') and adds
      # a unique number to the start to ensure every id is unique after mangling
      newseqid = `seqcount`+re.sub(r'[^\w]', "", seqid)
      seqcount += 1
      tmhmm_seq_id_mapping[newseqid] = seqid
      #seq.id = seqid
      seq.id = newseqid

      seq.seq = proteins[seqid]['seq']
      request.sequencedata.sequence.append(seq)
      sys.stderr.write(".")
      
    response = client.service.runService(request)

    sys.stderr.write("\n")
    
    #pollQueue
    job = client.factory.create('pollQueue.job')
    job.jobid = response.jobid
    response = client.service.pollQueue(job)
    retries = 0
    sys.stderr.write("# Waiting for TMHMM results ")
    while response.status != "FINISHED" and retries < 100:
      response = client.service.pollQueue(job)
      time.sleep(10 + (retries*2))
      retries += 1
      sys.stderr.write(".")
      
      # if something goes wrong, note it and skip TMHMM
      # by returning
      if response.status == "REJECTED" or \
         response.status == "UNKNOWN JOBID" or \
         response.status == "QUEUE DOWN" or \
         response.status == "FAILED":
        log_stderr("TMHMM(web) failed: '%s'" % (response.status))
        return proteins
        
    sys.stderr.write(" done !\n")
      
    #fetchResults
    done_job = client.factory.create('fetchResult.job')
    done_job.jobid = response.jobid
    result = client.service.fetchResult(done_job)
    
    if __DEBUG__: log_stderr(str(result))

    citation["name"] = result[0].method + " " + result[0].version
      
    for res in result.ann:
      #seqid = res.sequence.id
      seqid = tmhmm_seq_id_mapping[res.sequence.id]
      if 'tmhmm_helices' not in proteins[seqid]:
        proteins[seqid].update({
          'tmhmm_helices':[],
          'tmhmm_inner_loops':[],
          'tmhmm_outer_loops':[]
        })
      if len(res.annrecords) > 0:
        for segment in res.annrecords.annrecord:
          if segment.comment in mapping:
            tmhmmkey = mapping[segment.comment]
            proteins[seqid][tmhmmkey].append(\
                           (segment.range.begin, segment.range.end))
                           
      # for caching in the outfile
      if seqid not in tmhmm_dict:
        tmhmm_dict[seqid] = {}

      # extract a copy of results from proteins dictionary
      # ready to we written to cache file
      for k in mapping.values():
        tmhmm_dict[seqid][k] = proteins[seqid][k]

      tmhmm_dict[seqid]['program_name'] = citation['name']
   
  if __DEBUG__: print_proteins(proteins)
  # we store the minimal stuff in JSON format
  fh = open(outfile, 'w')
  fh.write(json.dumps(tmhmm_dict, separators=(',',':\n')))
  fh.close()    
  
  return proteins




# code loosely based on: https://portal.nordu.net/display/ndgfwiki/Python-Suds
# see: http://www.cbs.dtu.dk/ws/ws.php?entry=SignalP4

citation = {'ref': u"Petersen TN, Brunak S, von Heijne G, Nielsen H. "
                   u"SignalP 4.0: discriminating signal peptides from "
                   u"transmembrane regions. Nature methods 2011 "
                   u"Jan;8(10):785-6. \n"
                   u"<http://dx.doi.org/10.1038/nmeth.1701>",
            'name': "SignalP 4.0"
           }
          
__DEBUG__ = False

import sys, os, time, json
from suds.client import Client
from suds.bindings import binding 
import logging
from inmembrane.helpers import log_stderr

def annotate(params, proteins, \
             url = 'http://www.cbs.dtu.dk/ws/SignalP4/SignalP4_4_0_ws0.wsdl', \
             #url = 'http://www.cbs.dtu.dk/ws/SignalP/SignalP_3_1_ws0.wsdl', \
             batchsize = 2000, \
             force=False):
  if __DEBUG__:
    logging.basicConfig(level=logging.INFO)
    # soap messages (in&out) and http headers
    logging.getLogger('suds.client').setLevel(logging.DEBUG)             
      
  # grab the cached results if present
  outfile = "signalp_web.out"
  if not force and os.path.isfile(outfile):
    log_stderr("# -> skipped: %s already exists" % outfile)
    fh = open(outfile, 'r')
    annots = json.loads(fh.read())
    fh.close()
    for seqid in annots:
      proteins[seqid]['is_signalp'] = annots[seqid]['is_signalp']
      proteins[seqid]['signalp_cleave_position'] = \
        annots[seqid]['signalp_cleave_position']
    citation['name'] = annots[seqid]['program_name']
    return proteins
  
  log_stderr("# SignalP(web), %s > %s" % (params['fasta'], outfile))
  log_stderr("# SignalP(web): submitting in batches of %i sequences" % batchsize)
  
  seqids = proteins.keys()
  signalp_dict = {}
  while seqids:
    seqid_batch = seqids[0:batchsize]
    del seqids[0:batchsize]
    client = Client(url, cache=None)
    request=client.factory.create('runService.parameters')
    
    sys.stderr.write("# ")
    for seqid in seqid_batch:
      seq = client.factory.create('runService.parameters.sequencedata.sequence')
      seq.id = seqid
      seq.seq = proteins[seqid]['seq']
    
      # organism can be 'euk', 'gram+', 'siganlgram-'
      request.organism = params['signalp4_organism']
      # default for SignalP 4.0
      #request.method = 'best'
      # default for SignalP 3.1
      #request.method = 'nn+hmm'
      request.sequencedata.sequence.append(seq)
      sys.stderr.write(".")
      
    response = client.service.runService(request)

    sys.stderr.write("\n")
    
    #pollQueue
    job = client.factory.create('pollQueue.job')
    job.jobid = response.jobid
    response = client.service.pollQueue(job)
    retries = 0
    sys.stderr.write("# Waiting for SignalP results ")
    while response.status != "FINISHED" and retries < 100:
      response = client.service.pollQueue(job)
      time.sleep(10 + (retries*2))
      retries += 1
      sys.stderr.write(".")
      
      # if something goes wrong, note it and skip SignalP
      # by returning
      if response.status == "REJECTED" or \
         response.status == "UNKNOWN JOBID" or \
         response.status == "QUEUE DOWN" or \
         response.status == "FAILED":
        log_stderr("SignalP(web) failed: '%s'" % (response.status))
        return proteins
        
    sys.stderr.write(" done !\n")
      
    #fetchResults
    done_job = client.factory.create('fetchResult.job')
    done_job.jobid = response.jobid
    result = client.service.fetchResult(done_job)
    #log_stderr(str(result))

    # end of signal-nn
    
    citation["name"] = result[0].method + " " + result[0].version
      
    # TODO: the better way to do this would be to save the entire SOAP
    #       response returned by client.last_received() and then parse
    #       that upon plugin invocation (above) using suds.sax
    #       This way we save everything in the analysis, not just
    #       the details we are interested in right now
    for res in result.ann:
      seqid = res.sequence.id
      proteins[seqid]['signalp_cleave_position'] = 0
      proteins[seqid]['is_signalp'] = False
      if len(res.annrecords) > 0:
        # range.end - this is the last residue of the signal peptide if
        #  there is a cleavage site
        cleavage_site = int(res.annrecords.annrecord[0].range.end)
        if cleavage_site == 1: cleavage_site = 0
        proteins[seqid]['signalp_cleave_position'] = cleavage_site
        # from 'comment', "Y" or "N noTm" or "N TM" where "Y" means signal peptide
        signal_yn = res.annrecords[0][0].comment[0]
        if signal_yn == "Y":
          proteins[seqid]['is_signalp'] = True
        else:
          proteins[seqid]['is_signalp'] = False
            
      # for caching in the outfile
      if seqid not in signalp_dict:
        signalp_dict[seqid] = {}
      signalp_dict[seqid]['is_signalp'] = proteins[seqid]['is_signalp']
      signalp_dict[seqid]['signalp_cleave_position'] = \
        proteins[seqid]['signalp_cleave_position']
      signalp_dict[seqid]['program_name'] = citation['name']
    
  # we store the minimal stuff in JSON format
  fh = open(outfile, 'w')
  fh.write(json.dumps(signalp_dict, separators=(',',':\n')))
  fh.close()    
  
  return proteins




import os
import unittest
import sys

import inmembrane
import inmembrane.tests
from inmembrane import helpers
from inmembrane.plugins import lipop1

from inmembrane.tests.PluginTestBase import PluginTestBase

class TestLipoP(PluginTestBase):
  _plugin_name = "lipop1"

  def test_lipop1(self):
    if not self.params['lipop1_bin']:
      self.params['lipop1_bin'] = 'LipoP'

    lipop1.annotate(self.params, self.proteins)

    self.expected_output = {
        u'SPy_0252': True,
        u'SPy_2077': False, 
        u'SPy_0317': True,
        u'tr|Q9HYX8' : True,
    }
    
    for seqid in self.expected_output:
      self.assertEqual(
          self.expected_output[seqid], self.proteins[seqid]['is_lipop'])
    self.assertEqual(self.proteins[u'tr|Q9HYX8']['lipop_cleave_position'], 19)
    self.assertIn('lipop_im_retention_signal', self.proteins[u'tr|Q9HYX8'])
    self.assertTrue(self.proteins[u'tr|Q9HYX8']['lipop_im_retention_signal'])

if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-
from inmembrane.helpers import run, parse_fasta_header

citation = {'ref': u"Petersen TN, Brunak S, von Heijne G, Nielsen H. "
                   u"SignalP 4.0: discriminating signal peptides from "
                   u"transmembrane regions. Nature methods 2011 "
                   u"Jan;8(10):785-6. \n"
                   u"<http://dx.doi.org/10.1038/nmeth.1701>",
            'name': "SignalP 4.0"
           }

def annotate(params, proteins):
  for seqid in proteins:
    proteins[seqid]['is_signalp'] = False
    proteins[seqid]['signalp_cleave_position'] = None

  signalp4_out = 'signalp.out'
  cmd = '%(signalp4_bin)s -t %(signalp4_organism)s  %(fasta)s' % \
             params
  run(cmd, signalp4_out)

  for line in open(signalp4_out):
    if line.startswith("#"):
      continue
    words = line.split()
    seqid = parse_fasta_header(words[0])[0]
    proteins[seqid]['signalp_cleave_position'] = int(words[4])
    if (words[9] == "Y"):
      proteins[seqid]['is_signalp'] = True

  return proteins
    



# -*- coding: utf-8 -*-
# code loosely based on: https://portal.nordu.net/display/ndgfwiki/Python-Suds
# see: http://www.cbs.dtu.dk/ws/ws.php?entry=SignalP4

citation = {'ref': u"Agnieszka S. Juncker, Hanni Willenbrock, "
                   u"Gunnar Von Heijne, Søren Brunak, Henrik Nielsen, "
                   u"And Anders Krogh. (2003) Prediction of lipoprotein "
                   u"signal peptides in Gram-negative bacteria. Protein "
                   u"Science 12:1652–1662. \n"
                   u"<http://dx.doi.org/10.1110/ps.0303703>",
            'name': "LipoP 1.0.ws0"
           }
          
__DEBUG__ = False

import sys, os, time, json, re
from suds.client import Client
from suds.bindings import binding 
import logging
from inmembrane.helpers import log_stderr

def annotate(params, proteins, \
             #url = 'http://www.cbs.dtu.dk/ws/LipoP/LipoP_1_0_ws0.wsdl', \
             # we host our own fixed version of the WSDL for the moment
             url = "http://raw.github.com/boscoh/inmembrane/master/inmembrane/plugins/extra/LipoP_1_0_ws0.wsdl", \
             batchsize = 2000, \
             force=False):
  if __DEBUG__:
    logging.basicConfig(level=logging.INFO)
    # soap messages (in&out) and http headers
    logging.getLogger('suds.client').setLevel(logging.DEBUG)             
      
  # grab the cached results if present
  outfile = "lipop_web.out"
  if not force and os.path.isfile(outfile):
    log_stderr("# -> skipped: %s already exists" % outfile)
    fh = open(outfile, 'r')
    annots = json.loads(fh.read())
    fh.close()
    for seqid in annots:
      proteins[seqid]['is_lipop'] = annots[seqid]['is_lipop']
      proteins[seqid]['lipop_cleave_position'] = \
        annots[seqid]['lipop_cleave_position']
    
    citation['name'] = annots[seqid]['program_name']
    return proteins
  
  log_stderr("# LipoP(web), %s > %s" % (params['fasta'], outfile))
  log_stderr("# LipoP(web): submitting in batches of %i sequences" % batchsize)
  
  """
  # ensure schemas are correctly imported (workaround for broken schemas ..)
  from suds.xsd.doctor import ImportDoctor
  from suds.xsd.doctor import Import
  imp = Import("http://www.cbs.dtu.dk/ws/ws-common", location="http://www.cbs.dtu.dk/ws/common/ws_common_1_0b.xsd")
  imp.filter.add("http://www.cbs.dtu.dk/ws/WSLipoP_1_0_ws0")
  doctor = ImportDoctor(imp)
  client = Client(url, doctor=doctor, cache=None)
  #client = Client(url, plugins=[doctor], cache=None)
  """
  
  seqids = proteins.keys()
  lipop_dict = {}
  while seqids:
    seqid_batch = seqids[0:batchsize]
    del seqids[0:batchsize]
  
    client = Client(url, cache=None)
    
    request=client.factory.create('runService.parameters')
    
    # this is a horrible horrible workaround to account for the fact that
    # the lipop SOAP service returns null results if there is are certain
    # non-alphanumeric characters in the sequence id provided. horrible.
    lipop_seq_id_mapping = {}
    seqcount = 0
    
    sys.stderr.write("# ")
    for seqid in seqid_batch:
      seq = client.factory.create('runService.parameters.sequencedata.sequence')
      
      # workaround: removes any non-alphanumeric character (except '_') and adds
      # a unique number to the start to ensure every id is unique after mangling
      newseqid = `seqcount`+re.sub(r'[^\w]', "", seqid)
      seqcount += 1
      lipop_seq_id_mapping[newseqid] = seqid
      seq.id = newseqid
      #seq.id = seqid
      seq.seq = proteins[seqid]['seq']
    
      request.sequencedata.sequence.append(seq)
      sys.stderr.write(".")
        
    response = client.service.runService(request)

    sys.stderr.write("\n")
    
    #pollQueue
    job = client.factory.create('pollQueue.job')
    job.jobid = response.jobid
    response = client.service.pollQueue(job)
    retries = 0
    sys.stderr.write("# Waiting for LipoP results ")
    while response.status != "FINISHED" and retries < 100:
      response = client.service.pollQueue(job)
      time.sleep(10 + (retries*2))
      retries += 1
      sys.stderr.write(".")
      
      # if something goes wrong, note it and skip LipoP
      # by returning
      if response.status == "REJECTED" or \
         response.status == "UNKNOWN JOBID" or \
         response.status == "QUEUE DOWN" or \
         response.status == "FAILED":
        log_stderr("LipoP(web) failed: '%s'" % (response.status))
        return proteins
        
    sys.stderr.write(" done !\n")
      
    #fetchResults
    done_job = client.factory.create('fetchResult.job')
    done_job.jobid = response.jobid
    result = client.service.fetchResult(done_job)
    if __DEBUG__: log_stderr(str(result))

    
    citation["name"] = result[0].method + " " + result[0].version
      
    # TODO: the better way to do this would be to save the entire SOAP
    #       response returned by client.last_received() and then parse
    #       that upon plugin invocation (above) using suds.sax
    #       This way we save everything in the analysis, not just
    #       the details we are interested in right now
    for res in result.ann:
      #seqid = res.sequence.id
      seqid = lipop_seq_id_mapping[res.sequence.id]
      # init as if no lipop hit, may be reset below
      proteins[seqid]['is_lipop'] = False
      proteins[seqid]['lipop_cleave_position'] = 0
      proteins[seqid]['lipop_im_retention_signal'] = False
      if len(res.annrecords) > 0:
        # range.end - this is the first residue (Cys) of the mature protein if
        #  there is a SpII cleavage site
        for annrec in res.annrecords.annrecord:
          if annrec.feature == "CleavII":
            proteins[seqid]['lipop_cleave_position'] = int(annrec.range.begin)
            proteins[seqid]['is_lipop'] = True
            
            # check for an E.coli style inner membrane retention signal
            # Asp+2 to cleavage site. There are other apparent retention 
            # signals in E. coli and other gram- bacteria in addition to
            # the Asp+2 which we don't detect here (yet).
            # (Yamaguchi et al, 1988; Tokuda and Matsuyama, 2005 [review])
            plus_two = proteins[seqid]['lipop_cleave_position']+1
            if proteins[seqid]['seq'][plus_two] == 'D':
              proteins[seqid]['lipop_im_retention_signal'] = True
    
      # for caching in the outfile
      if seqid not in lipop_dict:
        lipop_dict[seqid] = {}
      lipop_dict[seqid]['is_lipop'] = proteins[seqid]['is_lipop']
      lipop_dict[seqid]['lipop_cleave_position'] = \
        proteins[seqid]['lipop_cleave_position']
      lipop_dict[seqid]['lipop_im_retention_signal'] = \
        proteins[seqid]['lipop_im_retention_signal']
      lipop_dict[seqid]['program_name'] = citation['name']

  # we store the minimal stuff in JSON format
  fh = open(outfile, 'w')
  fh.write(json.dumps(lipop_dict, separators=(',',':\n')))
  fh.close()    
  
  return proteins




# -*- coding: utf-8 -*-
from inmembrane.helpers import run, parse_fasta_header, dict_get

citation = {'ref': u"Agnieszka S. Juncker, Hanni Willenbrock, "
                   u"Gunnar Von Heijne, Søren Brunak, Henrik Nielsen, "
                   u"And Anders Krogh. (2003) Prediction of lipoprotein "
                   u"signal peptides in Gram-negative bacteria. Protein "
                   u"Science 12:1652–1662. \n"
                   u"<http://dx.doi.org/10.1110/ps.0303703>",
            'name': 'LipoP 1.0'
           } 


def annotate(params, proteins):
  """
  Uses LipoP to identify lipo-attachment signals in the protein. 
  The 'proteins' dictionary is annotated by adding two fields:
    - 'is_lipop' is a boolean indicating whether a signal is found or not
    - 'lipop_cleave_position' gives the position where the signal
        is cleaved and the protein is attached to a lipid

  Returns a reference to the proteins data structure.
  """

  lipop1_out = 'lipop.out'
  run('%(lipop1_bin)s %(fasta)s' % params, lipop1_out)

  # initialize fields in each protein
  for seqid in proteins:
    proteins[seqid]['is_lipop'] = False
    proteins[seqid]['lipop_cleave_position'] = None

  for l in open(lipop1_out):
    words = l.split()

    if 'SpII score' in l:
      seqid = parse_fasta_header(words[1])[0]
      if 'cleavage' in l:
        pair = words[5].split("=")[1]
        i = int(pair.split('-')[0])
      else:
        i = None
      proteins[seqid]['is_lipop'] = 'Sp' in words[2]
      proteins[seqid]['lipop_cleave_position'] = i

    # check for an E.coli style inner membrane retention signal
    # Asp+2 to cleavage site. There are other apparent retention 
    # signals in E. coli and other gram- bacteria in addition to
    # the Asp+2 which we don't detect here (yet).
    # (Yamaguchi et al, 1988; Tokuda and Matsuyama, 2005 [review])
    if dict_get(proteins[seqid], 'lipop_cleave_position'):
      plus_two = proteins[seqid]['lipop_cleave_position']+1
      if proteins[seqid]['seq'][plus_two] == 'D':
        proteins[seqid]['lipop_im_retention_signal'] = True
      
  return proteins



import os
import unittest
import sys

import inmembrane
import inmembrane.tests
from inmembrane import helpers
from inmembrane.plugins import tatfind_web

from inmembrane.tests.PluginTestBase import PluginTestBase

class TestTatfind(PluginTestBase):
  _plugin_name = "tatfind_web"
  
  def test_tatfind_web(self):
    tatfind_web.annotate(self.params, self.proteins)

    self.expected_output = {
      'sp|P31550': {
        'sequence_length': 327, 
        'name': 'THIB_ECOLI Thiamine-binding periplasmic protein OS=Escherichia coli (strain K12) GN=thiB PE=1 SV=2', 
        'seq': 'MLKKCLPLLLLCTAPVFAKPVLTVYTYDSFAADWGPGPVVKKAFEADCNCELKLVALEDGVSLLNRLRMEGKNSKADVVLGLDNNLLDAASKTGLFAKSGVAADAVNVPGGWNNDTFVPFDYGYFAFVYDKNKLKNPPQSLKELVESDQNWRVIYQDPRTSTPGLGLLLWMQKVYGDDAPQAWQKLAKKTVTVTKGWSEAYGLFLKGESDLVLSYTTSPAYHILEEKKDNYAAANFSEGHYLQVEVAARTAASKQPELAQKFLQFMVSPAFQNAIPTGNWMYPVANVTLPAGFEKLTKPATTLEFTPAEVAAQRQAWISEWQRAVSR', 
        'is_tatfind': False, 
      },
      'sp|P0AB06': {
        'sequence_length': 182, 
        'name': 'YCBK_ECOLI Uncharacterized protein ycbK OS=Escherichia coli (strain K12) GN=ycbK PE=4 SV=1', 
        'seq': 'MDKFDANRRKLLALGGVALGAAILPTPAFATLSTPRPRILTLNNLHTGESIKAEFFDGRGYIQEELAKLNHFFRDYRANKIKSIDPGLFDQLYRLQGLLGTRKPVQLISGYRSIDTNNELRARSRGVAKKSYHTKGQAMDFHIEGIALSNIRKAALSMRAGGVGYYPRSNFVHIDTGPARHW', 
        'is_tatfind': True, 
      },
      'sp|P31549': {
        'sequence_length': 536, 
        'name': 'THIP_ECOLI Thiamine transport system permease protein thiP OS=Escherichia coli (strain K12) GN=thiP PE=3 SV=2', 
        'seq': 'MATRRQPLIPGWLIPGVSATTLVVAVALAAFLALWWNAPQDDWVAVWQDSYLWHVVRFSFWQAFLSALLSVIPAIFLARALYRRRFPGRLALLRLCAMTLILPVLVAVFGILSVYGRQGWLATLCQSLGLEWTFSPYGLQGILLAHVFFNLPMASRLLLQALENIPGEQRQLAAQLGMRSWHFFRFVEWPWLRRQIPPVAALIFMLCFASFATVLSLGGGPQATTIELAIYQALSYDYDPARAAMLALLQMVCCLGLVLLSQRLSKAIAPGTTLLQGWRDPDDRLHSRICDTVLIVLALLLLLPPLLAVIVDGVNRQLPEVLAQPVLWQALWTSLRIALAAGVLCVVLTMMLLWSSRELRARQKMLAGQVLEMSGMLILAMPGIVLATGFFLLLNNTIGLPQSADGIVIFTNALMAIPYALKVLENPMRDITARYSMLCQSLGIEGWSRLKVVELRALKRPLAQALAFACVLSIGDFGVVALFGNDDFRTLPFYLYQQIGSYRSQDGAVTALILLLLCFLLFTVIEKLPGRNVKTD', 
        'is_tatfind': True, 
      },
    }

    #import helpers
    #helpers.print_proteins(self.proteins)
    
    self.assertEqual(self.proteins, self.expected_output)

if __name__ == '__main__':
  unittest.main()



import os
import unittest
import sys

import inmembrane
import inmembrane.tests
from inmembrane import helpers
from inmembrane.plugins import tmbetadisc_rbf_web

from inmembrane.tests.PluginTestBase import PluginTestBase

class TestTmbetadisc_rbf(PluginTestBase):
  _plugin_name = "tmbetadisc_rbf_web"
  
  def test_tmbetadisc_rbf_web(self): 
    tmbetadisc_rbf_web.annotate(self.params, self.proteins)
    
    self.expected_output = {
        'gi|107836852': {
          'sequence_length': 270, 
          'name': 'outer membrane protein HorF [Helicobacter pylori HPAG1]', 
          'seq': 'MKKFVVFKTLCLSVVLGNSLLAAEGSTEVQKQLEKPKEYKAVKGEKNAWYLGISYQVGQASQSVKNPPKSSEFNYPKFPVGKTDYLAVMQGLGLTVGYKQFFGEKRWFGARYYGFMDYGHAVFGANALTSDNGGACKLNEPCATKVGTMGNLSDMFTYGVGIDTLYNVINKEDASFGFFFGAQIAGNSWGNTTGAFLETKSPYKHTSYSLDPAIFQFLFNLGIRTHIGRHQEFDFGVKIPTINVYYFNHGNLSFTYRRQYSLYVGYRYNF', 
          'is_tmbetadisc_rbf': True, 
        },
        'gi|107837106': {
          'sequence_length': 326, 
          'name': 'heat shock protein [Helicobacter pylori HPAG1]', 
          'seq': 'MRARCSKKSSMRKEKIMTNFEKIIAQNRLKTNAVLTTYCVIFAFIGLLVDAIRINANDLGVALFKLITFQIFPTITTIMFLAAFVIIVVCIQNFSSIMLSGDEYKLIDTSKVLSSKENQIHRLLLELLEEAKLHFEPKLYIINAPYMNAFASGWDESNSLIALTSALIERLDKDELKAVIAHELSHIRHNDIRLTMCVGILSNIMLLVANFSVYFFMGNRKNSGANLARMILLLLQIVLPFLTLILQMYLSRTREYMADSGAAFLMHDNKPMIRALQKISNDYANNDYKGIDQNSTRSAAYLFNAEMFSTHPSIKNRIQSLSRRVI', 
          'is_tmbetadisc_rbf': False, 
        },
        'gi|107837101': {
          'sequence_length': 2417, 
          'name': 'vacuolating cytotoxin (VacA)-like protein [Helicobacter pylori HPAG1]', 
          'seq': 'MAFKKARLISRFISKGSFKLSKISKKIFTLNQILKCEKPLKCHKKTKSIKKLSNRNKSFLKASILLIGALGGLSHLRANECRYWSWSSWGYQDNIESGPNSPTHNSYCLFSSTQGSGTYYLNTLTTYSAGGASFTQKFNGGTLNVGGNIRFGGTGINGGDVGYITGTYDAQTINFNSSHLTTGNSYADGGGATLNFNAANNITINQASFDNSDAGAQKSYMNFKGSNIKVSGSSFKDDTDGGFSFSGNSNNSTISFNQTNFNQGTYNFSNSANLSFTNSAFNQGTYNFNSAQSVFENSSFNQGTYNFNSAQSVFENSAFNQGTYNFNGNASFDNDIFNQGTYNFNTSKVSFSGANTLNSSSPFASLKGSVSFGSDAVFNLNQTLNSNQTYDILTTNGTIQYGVYQSYLWDLINYKGDKAISHVEVGNNTYDVTFDINGQDETLQETFNKQSIITQFLGDDLQQQAQKTYQQDLSNSQSALNNAASDSKVANNDTGYTQSKNATVAKDAQGLENTNQKIQQDEQALEKDLAQIKQLANSTTGFNQQAFNQAQSTEQQDEQTLQNEENTFNIEQESLDKAIANAKHTSPTPSPTPTPTKHTAQNTPPNKVSPTPTPPTQNLPTTNVWNGVYWLQNQTYSKQGVYYIDPNLSGQSGQSANTLSTYTANLFGRSFGVNIQNGTLIIGNNTESVNDNGLIWIGHGGFGYITGTFSAANIYLTNNFKTGEGVSNSDGGGANITFKASDNITMDGLNYNDAETVTKMIQTGASQHSYAAFDALNNISVTNSSFSDMTWGKFSFSAKNISFSNASFSGFTNPGGSSVISANASNSLSFINSRLNGGAVYNLWANSLIFNNTQAVFNVLHSRGTSNFNATTQLLGNTNFTLSSQSLLNFNGDTTLQNNANITLGNKSQAAFKNSLTLDNDSNLSLDNQSVLNANGASAFNNQASLNIYNGSQATFKSLFFNGGTLSLNASSKLNASSASFSNNTTINLDDSVLSASNTSSLNANINFQGASQADFGGNTTINTASFNFDSASSLSFNNLTANGALNFNGYTPSLTKALMSVSGQFVLGNNGDINLSDINIFDNITKSVTYNILNAQKGITGISGANGYEKILFYGMKIQNATYSGNNNIQTWSFINPLNSSQIIQESIKNGDLTIEILNNPNSASNTIFNIAPELYNYQASKQNPTGYSYDYSDNQAGTYYLTSNIKGLFTPKGSQTPQTPGTYSPFNQPLNSLNIYNKGFSSENLKTLLGILSQNSATLKEMIESNQLDNITNINEVLQLLDKIKITQTQKQALLETINHLTDNINQTFNNGNLIIGATQDNVTNSTSSIWFGGNGYSSPCALDSATCSSFRNTYLGQLLGSTSPYLGYINADFKAKSIYITGTIGSGNAFESGGSADVTFQSTNNLVLNKANIEAQATDNIFNLLGQEGIDKIFNQGNLANVLSQMAMEKIKQAGGLGNFIENALSPLSKELPASLQDETLGQLIGQNNLDDLLNNSGVMNAIQNIISKKLSIFGNFVTPSIIENYLAKQSLKSMLDDKGLLNFIGGYIDASELSSILSVILKDITNPPTSLQKDIGVVANDLLNEFLGQDVIKKLESQELVNNIINNIISQGGLSGVYNQGLGSVLPPSLQNALKENDLGALLSPRGLHDFWQKGYFNFLSNGYVFVNNSSFSNATGGSLNFVANKSIIFDGDNTIDFSKYQGALIFASNDVSNINITTLNATNGLSLNAGLNNVSVQKGEICVNLANCPTTKNSSSTNSSVTPTNESLSVHANSFTFLGVIASNGAIDLSQVKNNSVIGTLNLNENATLQANNLTIANAFNNASNSTANINGDFTLNQQATLSTNASGLNVMGNFNSYGDLVFNLSHSVSHAIINAQGSATIMANNNNPLIQFNTSSKETGTYTLINSAKAIYYGYNDQITGGSSLDNYLKLYALIDINGKHMVMTDNGLTYNGQAVNIKDGGLVVGFKDSQNQYIYTSILYNKVKIAVSNDPINNLQAPTLKQYIAQIQGVQSVDSIDQVGGNQAINWLNKIFETKGSPLFAPYYLESHSTKDLTTIAGDIANTLEVIANPNFKNDATNILQINTYTQQMSRLAKLSDTSTFASADFHERLEALKNKRFADAIPNAMDVILKYSQRNRVKNNVWATGVGGASFINGGTGTLYGINVGYDRFIKGVIVGGYAAYGYSGFHGNITQSGSSNVNMGVYSRAFIKRSELTMSLNETWGYNKTFINSYDPLLSIINQSYRYDTWTTNAKINYGYDFMFKDKSVIFKPQVGLAYYYIGLSGLRGIMDDPIYNQFRANADPNKKSVLTINFALESRHYFNKNSYYFVIADVGRDLFINSMGDKMVRFIGNNTLSYRDGGRYNTFASIITGGEIRLFKTFYVNAGIGARFGLDYKDINITGNIGMRYAF', 
          'is_tmbetadisc_rbf': True, 
        },
        'gi|107836588': {
          'sequence_length': 240, 
          'name': 'hypothetical protein HPAG1_0390 [Helicobacter pylori HPAG1]', 
          'seq': 'MGLKKRAILWSLMGFYAGLNALDYDTLDPKYYKYIKYYKAYEDKEVEELIRDLKRANAKSGLILGINTGFFYNHEIMVRTNSSSITGNILNYLFAYGLRFGYQTFRPSFFARLVKPNIIGRRIYIQYYGGAPKKAGFGSVGFQSVMLNGDFLLDFPLPFVGKYLYMGGYMGLGLGVVAHGVNYTAEWGMSFNAGLALTVLEKNRIEFGFKILNNFPFLQSNSSKETWWGAIASIGYQYVF', 
          'is_tmbetadisc_rbf': True, 
        },
        'gi|107837107': {
          'sequence_length': 180, 
          'name': 'GTP cyclohydrolase I [Helicobacter pylori HPAG1]', 
          'seq': 'MENFFNQFFENIGEDKNREGLKETPKRVQELWKFLYKGYKEDPRVALKSAYFQGVCDEMIVAQNIEFYSTCEHHLLPFLGNISLGYIPKEKIVGISAIAKLIEIYSKRLQIQERLTTQITETFDEIIEPRGVIVVCEAKHLCMSMQGVQKQNAIIKTSVLRGLFKKDPKTRAEFMQLLKS', 
          'is_tmbetadisc_rbf': False, 
        },
      }

    #helpers.print_proteins(self.proteins)
    
    self.assertEqual(self.proteins, self.expected_output)

if __name__ == '__main__':
  unittest.main()



import os
import unittest
import sys

import inmembrane
import inmembrane.tests
from inmembrane import helpers
from inmembrane.plugins import hmmsearch3

from inmembrane.tests.PluginTestBase import PluginTestBase

class TestHmmsearch3(PluginTestBase):
  _plugin_name = "hmmsearch3"
  
  def test_hmmsearch3(self):
    self.params['hmm_profiles_dir'] = os.path.join(inmembrane.module_dir, 
                                       "protocols/gram_pos_profiles")
    hmmsearch3.annotate(self.params, self.proteins)

    self.expected_output = {
      'SPy_0191a': {
        'sequence_length': 69, 
        'hmmsearch': [], 
        'name': 'SPy_0191a from AE004092', 
        'seq': 'MSKQEVRDSLSTVVGDLSLTTRENQIGSLFLDVQSDEDFGFKVVKVLKSKGIVLNALDESVCGFKFVVE', 
      },
      'SPy_0128': {
        'sequence_length': 340, 
        'hmmsearch': ['LPxTG'], 
        'name': 'SPy_0128 from AE004092', 
        'seq': 'MKLRHLLLTGAALTSFAATTVHGETVVNGAKLTVTKNLDLVNSNALIPNTDFTFKIEPDTTVNEDGNKFKGVALNTPMTKVTYTNSDKGGSNTKTAEFDFSEVTFEKPGVYYYKVTEEKIDKVPGVSYDTTSYTVQVHVLWNEEQQKPVATYIVGYKEGSKVPIQFKNSLDSTTLTVKKKVSGTGGDRSKDFNFGLTLKANQYYKASEKVMIEKTTKGGQAPVQTEASIDQLYHFTLKDGESIKVTNLPVGVDYVVTEDDYKSEKYTTNVEVSPQDGAVKNIAGNSTEQETSTDKDMTITFTNKKDFEVPTGVAMTVAPYIALGIVAVGGALYFVKKKNA', 
      },
    }

    #helpers.print_proteins(self.proteins)
    
    self.assertEqual(self.proteins, self.expected_output)

if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-

citation = {'ref': u"Anders Krogh, Björn Larsson, Gunnar von Heijne and Erik "
                   u"L. L. Sonnhammer (2001) Predicting Transmembrane Protein "
                   u"Topology with a Hidden Markov Model: Application to "
                   u"Complete Genomes. J. Mol. Biol. 305:567-580. \n"
                   u"<http://dx.doi.org/10.1006/jmbi.2000.4315>",
            'name': "TMHMM 2.0"
           }

from inmembrane.helpers import run, parse_fasta_header


def annotate(params, proteins):
  """
  Runs THMHMM and parses the output files. Takes a standard 'inmembrane'
  params dictionary and a global proteins dictionary which it populates with
  results.
  
  In the current implementation, this function extracts and feeds sequences to tmhmm
  one by one via a temporary file.
  
  These keys are added to the proteins dictionary: 
    - 'tmhmm_helices', a list of tuples describing the first and last residue
      number of each transmembrane segment; 
    
    - 'tmhmm_scores', a list of confidence scores (floats) for each predicted 
      tm segment;
    
    - 'tmhmm_inner_loops', a list of tuples describing the first and last residue
       number of each predicted internal loop segment;
    
    - 'tmhmm_outer_loops', a list of tuples describing the first and last residue
       number of each predicted outer loop segment;
  """

  tmhmm_out = 'tmhmm.out'
  run('%(tmhmm_bin)s %(fasta)s' % params, tmhmm_out)

  seqid = None
  for i_line, l in enumerate(open(tmhmm_out)):
    if i_line == 0:
      continue
    words = l.split()
    if not words:
      continue

    if l.startswith("#"):
      seqid = parse_fasta_header(words[1])[0]
    else:
      seqid = parse_fasta_header(words[0])[0]
    if seqid is None:
      continue

    # initialize fields in proteins[seqid]
    if 'tmhmm_helices' not in proteins[seqid]:
      proteins[seqid].update({
        'tmhmm_helices':[],
        'tmhmm_inner_loops':[],
        'tmhmm_outer_loops':[]
      })

    if 'inside' in l:
      proteins[seqid]['tmhmm_inner_loops'].append(
          (int(words[-2]), int(words[-1])))
    if 'outside' in l:
      proteins[seqid]['tmhmm_outer_loops'].append(
          (int(words[-2]), int(words[-1])))
    if 'TMhelix' in l:
      proteins[seqid]['tmhmm_helices'].append(
          (int(words[-2]), int(words[-1])))

  return proteins




# This little bit of magic fills the __all__ list 
# with every plugin name, and means that calling:
# from plugins import *
# within inmembrane.py will import every plugin
import pkgutil

__all__ = []
for p in pkgutil.iter_modules(__path__):
  __all__.append(p[1])



import os, tempfile, shutil
import unittest
import sys

import inmembrane
import inmembrane.tests
from inmembrane import helpers

class PluginTestBase(unittest.TestCase):
  _plugin_name = ""
  def setUp(self):
    """
    Sets up a directory, a parameters dictionary and a proteins dictionary 
    object in preparation for running a test of a plugin.
    
    Creates a temporary directory (eg /tmp/.inmembrane_signalp_web_TleeRw ),
    copies the test input data (input.fasta) into it and changes the
    current working directory.
    
    Should be subclassed adding a method with the specific test(s) for
    the plugin. The subclass should set self._plugin_name to the name of
    the plugin.
    """
    self.test_data_dir = os.path.join(
       os.path.abspath(
       os.path.dirname(inmembrane.tests.__file__)), self._plugin_name)

    self.output_dir = tempfile.mkdtemp(prefix=".inmembrane_%s_" % (self._plugin_name))
    shutil.copyfile(os.path.join(self.test_data_dir, "input.fasta"), 
                    os.path.join(self.output_dir, "input.fasta"))
    os.chdir(self.output_dir)
    helpers.silence_log(True)
    
    self.params = inmembrane.get_params()
    self.params['fasta'] = "input.fasta"
    self.seqids, self.proteins = \
        helpers.create_proteins_dict(self.params['fasta'])



import os
import glob
import unittest
import sys

import inmembrane
import inmembrane.tests

from inmembrane import helpers
from inmembrane.plugins import bomp_web

from inmembrane.tests.PluginTestBase import PluginTestBase

class TestBomp(PluginTestBase):
  _plugin_name = "bomp_web"
  
  def test_bomp(self):
    self.expected_output = {
        u'gi|107837101': 3, 
        u'gi|107836588': 5, 
        u'gi|107836852': 5
    }

    self.output = bomp_web.annotate(self.params, self.proteins, force=True)


    self.assertEqual(self.expected_output, self.output)

if __name__ == '__main__':
  unittest.main()



__version__ = "0.92"
import helpers

import os, shutil
module_dir = os.path.abspath(os.path.dirname(__file__))

default_params_str = """{
  'fasta': '',
  'csv': '',
  'out_dir': '',
  'protocol': 'gram_pos', # 'gram_neg'
  
#### Signal peptide and transmembrane helix prediction
  'signalp4_bin': '',
# 'signalp4_bin': 'signalp',
  'lipop1_bin': '',
# 'lipop1_bin': 'LipoP',
  'tmhmm_bin': '',
#  'tmhmm_bin': 'tmhmm',
  'memsat3_bin': 'runmemsat',
  'helix_programs': ['tmhmm'],
# 'helix_programs': ['tmhmm', 'memsat3'],
  'terminal_exposed_loop_min': 50, # unused in gram_neg protocol
  'internal_exposed_loop_min': 100, # try 30 for gram_neg
  
#### Sequence similarity and motif prediction
  'hmmsearch3_bin': 'hmmsearch',
  'hmm_profiles_dir': '%(hmm_profiles)s',
  'hmm_evalue_max': 0.1,
  'hmm_score_min': 10,
  
#### Outer membrane beta-barrel predictors
  'barrel_programs': ['bomp', 'tmbetadisc-rbf'],
# 'barrel_programs': ['bomp', 'tmbetadisc-rbf', 'tmbhunt', 'tmbeta'],
  'bomp_clearly_cutoff': 3, # >= to this, always classify as an OM(barrel)
  'bomp_maybe_cutoff': 1, # must also have a signal peptide to be OM(barrel)
  'tmbhunt_clearly_cutoff': 0.95,
  'tmbhunt_maybe_cutoff': 0.5,
  'tmbetadisc_rbf_method': 'aadp', # aa, dp, aadp or pssm
}
"""

def get_params():
  from helpers import log_stderr
  """
  Gets the params dictionary that hold all the configuration
  information of the program. This is loaded from 'inmembrane.config'
  which should be found in the same place as the main binary.

  If 'inmembrane.config' is not a found, a default 'inmembrane.config'
  is generated from 'default_params_str'. The config file should
  be edited if the binaries are not available on the path, or have
  different names.
  """
  config = os.path.join(os.getcwd(), 'inmembrane.config')
  if not os.path.isfile(config):
    log_stderr("# Couldn't find inmembrane.config file")
    log_stderr("# So, will generate a default config " + config)
    abs_hmm_profiles = os.path.join(module_dir, 'hmm_profiles')
    default_str = default_params_str % \
        { 'hmm_profiles': abs_hmm_profiles }
    open(config, 'w').write(default_str)
  else:
    log_stderr("# Loading existing inmembrane.config")
  params = eval(open(config).read())
  return params

def init_output_dir(params):
  """
  Creates a directory for all output files and makes it the current 
  working directory. copies the input sequences into it as 'input.fasta'.
  """
  from helpers import dict_get
  
  if dict_get(params, 'out_dir'):
    base_dir = params['out_dir']
  else:
    base_dir = '.'.join(os.path.splitext(params['fasta'])[:-1])
    params['out_dir'] = base_dir
  if not os.path.isdir(base_dir):
    os.makedirs(base_dir)

  if not dict_get(params, 'csv'):
    basename = '.'.join(os.path.splitext(params['fasta'])[:-1])
    params['csv'] = basename + '.csv'
  params['csv'] = os.path.abspath(params['csv'])

  params['citations'] = os.path.join(params['out_dir'], 'citations.txt')
  params['citations'] = os.path.abspath(params['citations'])
  
  fasta = "input.fasta"
  shutil.copy(params['fasta'], os.path.join(base_dir, fasta))
  params['fasta'] = fasta

  shutil.copy(os.path.join(os.getcwd(), 'inmembrane.config'), base_dir)

  os.chdir(base_dir)


def import_protocol_python(params):
  """
  Some python magic that loads the desired protocol file
  encoded in the string 'params['protocol'] as a python file
  with the internal variable name 'protocol'. An appropriate
  python command is generated that is to be processed by
  the 'exec' function.
  """
  protocol_py = os.path.join(module_dir, 'protocols', params['protocol']+'.py')
  if not os.path.isfile(protocol_py):
    raise IOError("Couldn't find protcols/" + protocol_py)
  return 'import inmembrane.protocols.%s as protocol' % (params['protocol'])


def process(params):
  """
  Main program loop. Triggers the 'protocol' found in the params
  to annotate all proteins give the list of annotations needed by
  'protocol'. Then outputs to screen and a .csv file.
  """
  from helpers import dict_get, create_proteins_dict, log_stdout, log_stderr
  # will load all plugins in the plugins/ directory
  from inmembrane.plugins import *

  # initializations
  exec(import_protocol_python(params))
  init_output_dir(params)
  seqids, proteins = create_proteins_dict(params['fasta'])

  # TODO: ideally this loop needs to be run within the protocol,
  #       since for some protocols not all plugins
  #       will be run for every sequence, conditional
  #       on the outcome of a previous analysis
  #       eg. protocol.run(params, proteins)

  # annotates with external binaries as found in plugins
  for plugin_str in protocol.get_annotations(params):
    plugin = eval(plugin_str)
    plugin.annotate(params, proteins)
  
  # do protocol analysis on the results of the annotations
  for seqid in seqids:
    protein = proteins[seqid]
    protocol.post_process_protein(params, protein)
    log_stdout(protocol.protein_output_line(seqid, proteins))

  # print a summary table of classifications to stderr
  log_stderr(protocol.summary_table(params, proteins))

  # always write to biologist-friendly csv file
  f = open(params['csv'], 'w')
  for seqid in seqids:
    f.write(protocol.protein_csv_line(seqid, proteins))
  f.close()
  log_stderr("\n")
  log_stderr("Output written to %s" % (params['csv']))

  # TODO: citations for specific HMMs (PFAM etc ?)
  
  # write citations to a file and gracefully deal with plugins
  # without a citation defined
  import codecs
  import textwrap
  f = codecs.open(params['citations'], mode='w', encoding='utf-8')
  programs_used = []
  for program in protocol.get_annotations(params):
    plugin = eval(program)
    try:
      f.write(plugin.citation['name']+":\n")
      f.write(textwrap.fill(plugin.citation['ref']))
    except AttributeError:
      f.write("%s - no citation provided." % program)
    f.write("\n\n")
    try:
      programs_used.append(plugin.citation['name'])
    except (AttributeError, KeyError):
      programs_used.append(program)
    
  f.close()
  log_stderr("\n")
  log_stderr("This run used %s." % (", ".join(programs_used)) )
  log_stderr("References have been written to %s \n"
             "# - please cite as appropriate." % 
             (params['citations']) )
             
  return proteins



# -*- coding: utf-8 -*-
import os
import re
from inmembrane.helpers import seqid_to_filename, run, write_proteins_fasta

citation = {'ref': u"﻿Jones DT. Improving the accuracy of transmembrane "
                   u"protein topology prediction using evolutionary "
                   u"information. Bioinformatics (Oxford, England) "
                   u"2007 Mar;23(5):538-44. \n"
                   u"<http://dx.doi.org/10.1093/bioinformatics/btl677",
            'name': "MEMSAT3"
            }

def parse_memsat(protein, memsat_out):
  """
  Parses a MEMSAT3 output file for a FASTA sequence and
  annotates the protein dictionary. This is an auxillary function
  called by annotate, and the fields are described there.
  """

  f = open(memsat_out)
  l = f.readline()
  while l:
    l = f.readline()

    if l == "FINAL PREDICTION\n":
      f.readline()
      l = f.readline()
      s = l.split(":")

      while re.match("\d", l[0]):
        tokens = s[1].strip().split()
        tok_offset = 0
        if len(tokens) > 2:
          tok_offset = 1
          side_of_membrane_nterminus = tokens[0][1:-1] # 'in' or 'out'
        i = int(tokens[tok_offset].split('-')[0])
        j = int(tokens[tok_offset].split('-')[1])
        protein['memsat3_helices'].append((i, j))
        score = float(tokens[1+tok_offset][1:-1])
        protein['memsat3_scores'].append(score)
        l = f.readline()
        s = l.split(":")
      f.readline()
      
      # record inner and outer loops
      inner_loops = protein['memsat3_inner_loops']
      outer_loops = protein['memsat3_outer_loops']
      sequence_length = protein['sequence_length']
      if side_of_membrane_nterminus == 'out':
        loops = outer_loops
      elif side_of_membrane_nterminus == 'in':
        loops = inner_loops
      loop_start = 1

      # figure out helices
      for tm in protein['memsat3_helices']:
        loop_end = tm[0] - 1
        loop = (loop_start, loop_end)
        loops.append((loop_start, loop_end))
        if loops == outer_loops:
          loops = inner_loops
        else:
          loops = outer_loops
        loop_start = tm[1] + 1     

      # capture C-terminal loop segment
      loop_start = tm[1]+1
      loop_end = sequence_length
      loops.append((loop_start, loop_end))

  f.close()


def has_transmembrane_in_globmem(globmem_out):
  for l in open(globmem_out):
    if "Your protein is probably not a transmembrane protein" in l:
      return False
  return True

        
def annotate(params, proteins):
  """
  Runs MEMSAT3 and parses the output files. Takes a standard 'inmembrane'
  params dictionary and a global proteins dictionary which it populates with
  results.
  
  In the current implementation, this function extracts and feeds sequences to MEMSAT3
  one by one via a temporary file.
  
  These keys are added to the proteins dictionary: 
    - 'memsat3_helices', a list of tuples describing the first and last residue
      number of each transmembrane segment; 
    
    - 'memsat3_scores', a list of confidence scores (floats) for each predicted 
      tm segment;
    
    - 'memsat3_inner_loops', a list of tuples describing the first and last residue
       number of each predicted internal loop segment;
    
    - 'memsat3_outer_loops', a list of tuples describing the first and last residue
       number of each predicted outer loop segment;
  """

  for seqid in proteins:
    protein = proteins[seqid]

    # initialize the protein data structure
    protein.update({
      'memsat3_scores':[],
      'memsat3_helices':[],
      'memsat3_inner_loops':[],
      'memsat3_outer_loops':[]
    })

    # write seq to single fasta file
    single_fasta = seqid_to_filename(seqid) + '.fasta'
    if not os.path.isfile(single_fasta):
      write_proteins_fasta(single_fasta, proteins, [seqid])

    memsat_out = single_fasta.replace('fasta', 'memsat')
    run('%s %s' % (params['memsat3_bin'], single_fasta), memsat_out)

    globmem_out = single_fasta.replace('fasta', 'globmem')
    if has_transmembrane_in_globmem(globmem_out):
      parse_memsat(protein, memsat_out)



      



import os
import unittest
import sys

import inmembrane
import inmembrane.tests
from inmembrane import helpers
from inmembrane.plugins import memsat3

from inmembrane.tests.PluginTestBase import PluginTestBase

class TestMemsat3(PluginTestBase):
  _plugin_name = "memsat3"
  
  def test_memsat3(self):
    memsat3.annotate(self.params, self.proteins)

    self.expected_output = {
      'SPy_1392': {
        'memsat3_scores': [17.99, 23.11, 27.57, 23.5, 10.29, 9.08, 34.04, 32.19, 28.49, 17.9, 19.18, 25.52], 
        'memsat3_helices': [(8, 27), (44, 68), (73, 92), (102, 126), (132, 150), (153, 172), (220, 243), (258, 281), (286, 309), (312, 334), (345, 368), (373, 392)], 
        'name': 'SPy_1392 from AE004092', 
        'seq': 'MEKTKRYIIATAGILLHLMLGSTYAWSVYRNPILQETGWDQAPVAFAFSLAIFCLGLSAAFMGNLVEQYGPRLTGTVSAILYASGNMLTGLAIDRKEIWLLYIGYGVIGGLGLGAGYITPISTIIKWFPDKRGMATGFAIMGFGFASLLTSPIAQWLIETEGLVATFYLLGLIYLIVMLFASQLIIKPTAAEIAILDKKRLQNNSYLIEGMTAKEALKTKSFYCLWVILFINITCGLGLISVVAPMAQDLTGMSPEMSAIVVGAMGIFNGFGRLVWASLSDYIGRRVTVILLFLVSIIMTISLIFAHSSLIFMISIATLMTCYGAGFSLIPPYLSDLFGAKELATLHGYILTAWAIAALTGPMLLSITVEWTHNYLLTLCVFIVLYILGLMVALRLKK', 
        'memsat3_outer_loops': [(28, 43), (93, 101), (151, 152), (244, 257), (310, 311), (369, 372)], 
        'sequence_length': 398, 
        'memsat3_inner_loops': [(1, 7), (69, 72), (127, 131), (173, 219), (282, 285), (335, 344), (393, 398)], 
      },
      'SPy_1379': {
        'memsat3_scores': [15.02, 20.87, 22.47, 28.1], 
        'memsat3_helices': [(35, 59), (249, 272), (286, 309), (323, 346)], 
        'name': 'SPy_1379 from AE004092', 
        'seq': 'MTIIIMDSNSAHETDNLSVSFLNFCYNSLMKRHFLLLTFYLFLTGLTAGLVAFILTKAIHLIQSLSFGFSQGSFSTMIASVPPQRRALSLLFAGLLAGLGWHLLAKKGKDIQSIQQIIQDDISFSPWTQFWHGWLQLTTVSMGAPVGREGASREVAVTLTSLWSQRCNLSKADQKLLLACASGAALGAVYNAPLATILFILEAILNRWSLKNIYAACLTSYVAVETVALLQGRHEIQYLMPQQHWTLGTLIGSVLAGLILSLFAHAYKHLLKHLPKADAKSQWFIPKVLIAFSLIAGLSIFFPEILGNGKAGLLFFLHEEPHLSYISWLLVAKAVAISLVFASGAKGGKIAPSMMLGGASGLLLAILSQYLIPLSLSNTLAIMVGATIFLGVINKIPLAAPVFLVEITGQSLLMIIPLALANLIFYFSYQFYRFILK', 
        'memsat3_outer_loops': [(60, 248), (310, 322)], 
        'sequence_length': 437, 
        'memsat3_inner_loops': [(1, 34), (273, 285), (347, 437)], 
      },
      'SPy_1949': {
        'memsat3_scores': [11.44, 13.03, 19.61, 22.59, 8.81, 12.86, 18.54, 17.77], 
        'memsat3_helices': [(16, 35), (40, 59), (89, 112), (135, 159), (220, 243), (261, 283), (310, 334), (337, 356)], 
        'name': 'SPy_1949 from AE004092', 
        'seq': 'MEALLSFIRDILKEPAFLMGLIAFAGLVALKTPAHKVLTGTLGPILGYLMLVAGAGVIVTNLDPLAKLIEHGFSITGVVPNNEAVTSVAQKILGVETMSILVVGLLLNLAFARFTRFKYIFLTGHHSFFMACLLSAVLGAVGFKGSLLIILDGFLLGAWSAISPAIGQQYTLKVTDGDEIAMGHFGSLGYYLSAWVGSKVGKDSKDTEDLQISEKWSFLRNTTISTGLIMVIFYLVATVASVLRNASVAEELAAGQNPFIFAIKSGLTFAVGVAIVYAGVRMILADLIPAFQGIANKLIPNAIPAVDCAVFFPYAPTAVIIGFASSFVGGLLGMLILGVAGGVLIIPGMVPHFFCGATAEIFGNSTGGRRGAMIGASLMAYYSPSCQPCFYLYLVNLVFQTRPLEMWISVF', 
        'memsat3_outer_loops': [(1, 15), (60, 88), (160, 219), (284, 309), (357, 411)], 
        'sequence_length': 411, 
        'memsat3_inner_loops': [(36, 39), (113, 134), (244, 260), (335, 336)], 
      },
    }

    for seqid in self.expected_output:
      self.assertTrue(seqid in self.proteins)
      for prop in self.expected_output[seqid]:
        self.assertEqual(
          self.expected_output[seqid][prop], self.proteins[seqid][prop])

if __name__ == '__main__':
  unittest.main()



#
# Common helper functions for inmembrane
#

import inmembrane
import os, subprocess, sys
import textwrap

LOG_SILENT = False

def dict_get(this_dict, prop):
  """
  Useful helper function to get values from dicts. Takes in
  the possibility that the key may not exist, and in that
  case returns False. Makes it easier to write code to avoid
  handling the case of missing keys.
  """
  if prop not in this_dict:
    return False
  return this_dict[prop]
  

def run_with_output(cmd):
  """
  Runs an external program as a child process and captures
  the input. May not work with external programs that include
  complicated levels of shells.
  """
  p = subprocess.Popen(
      cmd, shell=True, stdout=subprocess.PIPE, 
      stderr=subprocess.PIPE)
  return p.stdout.read()


def run(cmd, out_file=None):
  """
  Wrapper function to run external program so that existence
  of binary can be checked and the output directed to a specific 
  file.
  """
  log_stderr("# " + cmd + " > " + out_file)
  if os.path.isfile(out_file) and (out_file != None):
    log_stderr("# -> skipped: %s already exists" % out_file)
    return
  binary = cmd.split()[0]
  is_binary_there = False
  if os.path.isfile(binary):
    is_binary_there = True
  if run_with_output('which ' + binary):
    is_binary_there = True
  if not is_binary_there:
    raise IOError("Couldn't find executable binary '" + binary + "'")
  if out_file:
    os.system(cmd + " > " + out_file)
  else:
    os.system(cmd)


def silence_log(b):
  """
  Turns on logging silent mode w.r.t. to log_stderr and log_stdout
  """
  global LOG_SILENT
  LOG_SILENT = b


def log_stderr(s):
  """
  Wrapper for all stderr out. Allows future customization.
  """
  if LOG_SILENT:
    return
  if s and s[-1] != "\n":
    s += "\n"
  if not s.startswith("#"):
    s = "# " + s
  sys.stderr.write(s)


def log_stdout(s):
  """
  Wrapper for all stderr out. Allows future customization.
  """
  if LOG_SILENT:
    return
  print s


def parse_fasta_header(header):
  """
  Parses a FASTA format header (with our without the initial '>') and returns a
  tuple of sequence id and sequence name/description.
  
  If NCBI SeqID format (gi|gi-number|gb|accession etc, is detected
  the first id in the list is used as the canonical id (see see
  http://www.ncbi.nlm.nih.gov/books/NBK21097/#A631 ).
  """
  # check to see if we have an NCBI-style header
  if header[0] == '>':
    header = header[1:]
  if header.find("|") != -1:
    tokens = header.split('|')
    # "gi|ginumber|gb|accession bla bla" becomes "gi|ginumber"
    seqid = "%s|%s" % (tokens[0], tokens[1].split()[0])
    name = tokens[-1:][0].strip()
  # otherwise just split on spaces & hope for the best
  else:
    tokens = header.split()
    seqid = tokens[0]
    name = header[0:-1].strip()
  
  return seqid, name
  

def seqid_to_filename(seqid):
  """
  Makes a sequence id filename friendly.
  (eg, replaces '|' with '_')
  """
  return seqid.replace("|", "_")


def create_proteins_dict(fasta):
  """
  The main data-structure used in inmembrane is the proteins dictionary,
  which is initialized here from a source FASTA file. Also returns a list 
  of seqID's in the same order as that in the source FASTA file.
  """
  seqids = []
  seqid = None
  proteins = {}
  for l in open(fasta):
    if l.startswith(">"):
      seqid, name = parse_fasta_header(l)
      seqids.append(seqid)
      proteins[seqid] = {
        'seq':"",
        'name':name,
      }
      continue
    if seqid is not None:
      words = l.split()
      if words:
        proteins[seqid]['seq'] += words[0]
    proteins[seqid]['sequence_length'] = len(proteins[seqid]['seq'])
  return seqids, proteins
  

def print_proteins(proteins):
  """
  Prints out the proteins dictionary in a formatted 
  manner that is also Python-eval compatible.
  """
  print "{"
  for seqid in proteins:
    print "  '%s': {" % seqid
    for key, value in proteins[seqid].items():
      print "    '%s': %s, " % (key, repr(value))
    print "  },"
  print "}"
  # Standard Library alternative 
  # import pprint
  # pp = pprint.PrettyPrinter(indent=4)
  # print pp.pformat(proteins)
 
  
def write_proteins_fasta(
    fasta_filename, proteins, seqids, width=50):
  """
  Creates a fasta file of the sequences of a subset of the proteins.
  """
  f = open(fasta_filename, "w")
  for seqid in seqids:
    seq_wrap = textwrap.fill(proteins[seqid]['seq'], width)
    f.write(">%s\n%s\n" % (proteins[seqid]['name'], seq_wrap))
  f.close()


def chop_nterminal_peptide(protein, i_cut):
  """
  Finds all transmembrane fields in protein ("*_loops" and "*_helices")
  and deletes an N-terminal section of the protein indicated by i_cut.

  Assuming the topology of inner and outer loops remain the same, this
  may delete a certain number of elements at the N-terminus. Allows a
  simple way of removing lipo-protein and secretion-protein signals from
  the evaluation of the outer_loops of the protein.
  """
  protein['sequence_length'] -= i_cut
  for prop in protein:
    if '_loops' in prop or '_helices' in prop:
      sses = protein[prop]
      for i in range(len(sses)):
        j, k = sses[i]
        sses[i] = (j - i_cut, k - i_cut)
  for prop in protein:
    if '_loops' in prop or '_helices' in prop:
      sses = protein[prop]
      for i in reversed(range(len(sses))):
        j, k = sses[i]
        # tests if this loop or TM-helix has been cut out
        if j<=0 and k<=0:
          del sses[i]
        # otherewise, neg value means loop or TM-helix is at the new N-terminal
        elif j<=0 and k>0:
          sses[i] = (1, k)
          # if a TM-helix gets cut in half and becomes a new N-terminal,
          # convert the remaining residues to a loop and remove the helix
          if '_helices' in prop:
            program = prop.split('_')[0]
            for x in protein:
              if x == '%s_loops' % program:
                new_N_loop = protein[x][0]
                new_N_loop[0] = 1
            del sses[i]
            

def eval_surface_exposed_loop(
    sequence_length, n_transmembrane_region, outer_loops, 
    terminal_exposed_loop_min, internal_exposed_loop_min):
  """
  This is the key algorithm in SurfG+ to identify Potentially Surface
  Exposed proteins. It evaluates all loops that poke out of the periplasmic
  side of the Gram+ bacterial membrane and tests if the loops are long
  enough to stick out of the peptidoglycan layer to be cleaved by proteases
  in a cell-shaving experiment.

  Returns True if any outer loop, or the N- or C-terminii are
  longer than the given thresholds.
  """

  if n_transmembrane_region == 0:
    # treat protein as one entire exposed loop
    return sequence_length >= terminal_exposed_loop_min

  if not outer_loops:
    return False

  loop_len = lambda loop: abs(loop[1]-loop[0]) + 1

  # if the N-terminal loop sticks outside
  if outer_loops[0][0] == 1:
    nterminal_loop = outer_loops[0]
    del outer_loops[0]
    if loop_len(nterminal_loop) >= terminal_exposed_loop_min:
      return True

  # if the C-terminal loop sticks outside
  if outer_loops:
    if outer_loops[-1][-1] == sequence_length:
      cterminal_loop = outer_loops[-1]
      del outer_loops[-1]
      if loop_len(cterminal_loop) >= terminal_exposed_loop_min:
        return True

  # test remaining outer loops for length
  for loop in outer_loops:
    if loop_len(loop) >= internal_exposed_loop_min:
      return True

  return False


def clean_directory(top, excluded_files):
  """
  Deletes an entire directory tree, equivalent to rm -rf <top>, 
  with an excluded file list.
  """
  for root, dirs, files in os.walk(top, topdown=False):
    for name in files:
      if name not in excluded_files:
        os.remove(os.path.join(root, name))
    for name in dirs:
      os.rmdir(os.path.join(root, name))



# -*- coding: utf-8 -*-

citation = {'ref': "http://hmmer.org", 'name' : 'HMMER 3.0'}

import os
import glob
from inmembrane.helpers import log_stderr, run, parse_fasta_header


def annotate(params, proteins):
  """
  Returns a reference to the proteins data structure.

  Uses HMMER to identify sequence motifs in proteins. This function
  annotates the proteins with:
    - 'hmmsearch': a list of motifs that are found in the protein. The
       motifs correspond to the basename of the .hmm files found in the directory
       indicated by the 'hmm_profiles_dir' field of 'params'.
  """

  log_stderr("# Searching for HMMER profiles in " + params['hmm_profiles_dir'])

  file_tag = os.path.join(params['hmm_profiles_dir'], '*.hmm')
  for hmm_profile in glob.glob(file_tag):
    params['hmm_profile'] = hmm_profile

    hmm_profile = os.path.basename(params['hmm_profile'])
    hmm_name = hmm_profile.replace('.hmm', '')
    hmmsearch3_out = 'hmm.%s.out' % hmm_name

    cmd = '%(hmmsearch3_bin)s -Z 2000 -E 10 %(hmm_profile)s %(fasta)s' % params
    run(cmd, hmmsearch3_out)

    # init proteins data structure with blank hmmsearch field first
    for seqid in proteins:
      if 'hmmsearch' not in proteins[seqid]:
        proteins[seqid]['hmmsearch'] = []

    # parse the hmmsearch output file
    seqid = None
    for l in open(hmmsearch3_out):
      words = l.split()

      if l.startswith(">>"):
        seqid = parse_fasta_header(l[3:])[0]
        continue

      if seqid is None:
        continue

      if 'conditional E-value' in l:
        evalue = float(words[-1])
        score = float(words[-5])
        if evalue <= params['hmm_evalue_max'] and \
            score >= params['hmm_score_min']:
          proteins[seqid]['hmmsearch'].append(hmm_name)

  return proteins



# -*- coding: utf-8 -*-

citation = {'ref': u"Rose, R.W., T. Brüser,. J. C. Kissinger, and M. "
                   u"Pohlschröder. 2002. Adaptation of protein secretion "
                   u"to extremely high salt concentrations by extensive use "
                   u"of the twin arginine translocation pathway. Mol. Microbiol."
                   u"5: 943-950 \n"
                   u"<http://dx.doi.org/10.1046/j.1365-2958.2002.03090.x>",
            'name': "TatFind 1.4"
           }

__DEBUG__ = False

import sys, os, time, StringIO

import twill
from twill.commands import find, formfile, follow, fv, go, show, \
                             showforms, showlinks, submit, agent
from inmembrane.helpers import log_stderr, parse_fasta_header

def parse_tatfind_output(output, proteins):
  """
  Parses the TatFind HTML output (file-like object or a list of strings) 
  an uses it to annotate and return an associated 'proteins' data structure.
  """
  for l in output:
    if "Results for" in l:
      seqid = l.split("Results for ")[1].split(":")[:-1][0]
      # parse id string to bring it to our format
      seqid, unused = parse_fasta_header(seqid)
      # "TRUE" or "FALSE"
      tat_pred = l.split("Results for ")[1].split(":")[-1:][0].strip()
      if tat_pred == "TRUE":
        proteins[seqid]["is_tatfind"] = True
      else:
        proteins[seqid]["is_tatfind"] = False
        
  return proteins

def annotate(params, proteins, \
             url="http://signalfind.org/tatfind.html", force=False):
  """
  Interfaces with the TatFind web service at (http://signalfind.org/tatfind.html) 
  to predict if protein sequences contain Twin-Arginine Translocation (Tat) 
  signal peptides.
  """
  # set the user-agent so web services can block us if they want ... :/
  python_version = sys.version.split()[0]
  agent("Python-urllib/%s (twill; inmembrane)" % python_version)
  
  outfn = 'tatfind.out'
  log_stderr("# TatFind(web) %s > %s" % (params['fasta'], outfn))
  
  if not force and os.path.isfile(outfn):
    log_stderr("# -> skipped: %s already exists" % outfn)
    fh = open(outfn, 'r')
    proteins = parse_tatfind_output(fh, proteins)
    fh.close()
    return proteins
  
  # dump extraneous output into this blackhole so we don't see it
  if not __DEBUG__: twill.set_output(StringIO.StringIO())
  
  go(url)
  if __DEBUG__: showforms()
  formfile("1", "seqFile", params["fasta"])
  submit()
  if __DEBUG__: show()
  
  tatfind_output = show()
  if __DEBUG__: log_stderr(tatfind_output)
  
  # write raw TatFind output to a file
  fh = open(outfn, 'w')
  fh.write(tatfind_output)
  fh.close()
  
  proteins = parse_tatfind_output(tatfind_output.split("\n"), proteins) 
  
  return proteins



#!/usr/bin/env python

params = {
  'fasta': 'examples/surfg/AE004092.fasta',
  'out_dir': '',
  'csv': 'examples/surfg/AE004092.csv',
  'protocol': 'gram_pos',
  'signalp4_bin': 'signalp',
  'signalp4_organism': 'gram+',
  'lipop1_bin': 'LipoP',
  'tmhmm_bin': 'tmhmm',
  'helix_programs': ['tmhmm'],
  'memsat3_bin': 'runmemsat',
  'hmmsearch3_bin': 'hmmsearch',
  'hmm_profiles_dir': 'protocols/gram_pos_profiles',
  'hmm_evalue_max': 0.1,
  'hmm_score_min': 10,
  'terminal_exposed_loop_min': 50,
  'internal_exposed_loop_min': 100,
}

import inmembrane
inmembrane.process(params)



# -*- coding: utf-8 -*-

citation = {'ref': u"Garrow, A.G., Agnew, A. and Westhead, D.R. TMB-Hunt: An "
                   u"amino acid composition based method to screen proteomes "
                   u"for beta-barrel transmembrane proteins. BMC "
                   u"Bioinformatics, 2005, 6: 56 \n "
                   u"<http://dx.doi.org/10.1186/1471-2105-6-56>",
            'name': "TMB-HUNT"
           }

__DEBUG__ = False

import sys, os, time, StringIO

import twill
from twill.commands import find, formfile, follow, fv, go, show, \
                             showforms, showlinks, submit, agent
                             
from inmembrane.helpers import log_stderr, parse_fasta_header


def annotate(params, proteins, \
             force=False):
  """
  Uses the TMB-HUNT web service 
  (http://bmbpcu36.leeds.ac.uk/~andy/betaBarrel/AACompPred/aaTMB_Hunt.cgi) to
  predict if proteins are outer membrane beta-barrels.
  
  NOTE: In my limited testing, TMB-HUNT tends to perform very poorly in
        terms of false positives and false negetives. I'd suggest using only
        BOMP.
  """
  # TODO: automatically split large sets into multiple jobs
  #       TMB-HUNT will only take 10000 seqs at a time
  if len(proteins) >= 10000:
    log_stderr("# ERROR: TMB-HUNT(web): can't take more than 10,000 sequences.")
    return
  
  # set the user-agent so web services can block us if they want ... :/
  python_version = sys.version.split()[0]
  agent("Python-urllib/%s (twill; inmembrane)" % python_version)
  
  out = 'tmbhunt.out'
  log_stderr("# TMB-HUNT(web) %s > %s" % (params['fasta'], out))
  
  if not force and os.path.isfile(out):
    log_stderr("# -> skipped: %s already exists" % out)
    return parse_tmbhunt(proteins, out)
  
  # dump extraneous output into this blackhole so we don't see it
  if not __DEBUG__: twill.set_output(StringIO.StringIO())
  
  go("http://bmbpcu36.leeds.ac.uk/~andy/betaBarrel/AACompPred/aaTMB_Hunt.cgi")
  if __DEBUG__: showforms()

  # read up the FASTA format seqs
  fh = open(params['fasta'], 'r')
  fasta_seqs = fh.read()
  fh.close()
  
  # fill out the form
  fv("1", "sequences", fasta_seqs)

  submit()
  if __DEBUG__: showlinks()

  # small jobs will lead us straight to the results, big jobs
  # go via a 'waiting' page which we skip past if we get it
  job_id = None
  try:
    # we see this with big jobs
    result_table_url = follow("http://www.bioinformatics.leeds.ac.uk/~andy/betaBarrel/AACompPred/tmp/tmp_output.*.html")
    job_id = result_table_url.split('tmp_output')[-1:][0].split('.')[0]
  except:
    # small jobs take us straight to the html results table
    pass

  # parse the job_id from the url, since due to a bug in
  # TMB-HUNT the link on the results page from large jobs is wrong
  if not job_id: job_id = follow("Full results").split('/')[-1:][0].split('.')[0]
  log_stderr("# TMB-HUNT(web) job_id is: %s <http://www.bioinformatics.leeds.ac.uk/~andy/betaBarrel/AACompPred/tmp/tmp_output%s.html>" % (job_id, job_id))
  
  # polling until TMB-HUNT finishes
  # TMB-HUNT advises that 4000 sequences take ~10 mins
  # we poll a little faster than that
  polltime = (len(proteins)*0.1)+2
  while True:
    log_stderr("# TMB-HUNT(web): waiting another %i sec ..." % (polltime))
    time.sleep(polltime)
    try:
      go("http://bmbpcu36.leeds.ac.uk/~andy/betaBarrel/AACompPred/tmp/%s.txt" % (job_id))
      break
    except:
      polltime = polltime * 2
      
    if polltime >= 7200: # 2 hours
      log_stderr("# TMB-HUNT error: Taking too long.")
      return
    
  txt_out = show()
  
  # write raw TMB-HUNT results
  fh = open(out, 'w')
  fh.write(txt_out)
  fh.close()
  
  return parse_tmbhunt(proteins, out)


def parse_tmbhunt(proteins, out):
  """
  Takes the filename of a TMB-HUNT output file (text format)
  & parses the outer membrane beta-barrel predictions into the proteins dictionary.
  """
  # parse TMB-HUNT text output
  tmbhunt_classes = {}
  for l in open(out, 'r'):
    #inmembrane.log_stderr("# TMB-HUNT raw: " + l[:-1])
    if l[0] == ">":
      # TMB-HUNT munges FASTA ids by making them all uppercase,
      # so we find the equivalent any-case id in our proteins list
      # and use that. ugly but necessary.
      seqid, desc = parse_fasta_header(l)
      for i in proteins.keys():
        if seqid.upper() == i.upper():
          seqid = i
          desc = proteins[i]['name']
        
      probability = None
      classication = None
      tmbhunt_classes[seqid] = {}
    if l.find("Probability of a NON-BETA BARREL protein with this score:") != -1:
      # we convert from probability of NON-BARREL to probability of BARREL
      probability = 1 - float(l.split(":")[1].strip())
    if l[0:11] == "Conclusion:":
      classication = l.split(":")[1].strip()
      if classication == "BBMP":
        tmbhunt_classes[seqid]['tmbhunt'] = True
        tmbhunt_classes[seqid]['tmbhunt_prob'] = probability
        
        proteins[seqid]['tmbhunt'] = True
        proteins[seqid]['tmbhunt_prob'] = probability
        
      elif classication == "Non BBMP":
        tmbhunt_classes[seqid]['tmbhunt'] = False
        tmbhunt_classes[seqid]['tmbhunt_prob'] = probability
        
        proteins[seqid]['tmbhunt'] = False
        proteins[seqid]['tmbhunt_prob'] = probability
  
  #inmembrane.log_stderr(str(tmbhunt_classes))
  return tmbhunt_classes



import os
import unittest
import sys

import inmembrane
import inmembrane.tests
from inmembrane import helpers
from inmembrane.plugins import signalp4

from inmembrane.tests.PluginTestBase import PluginTestBase

class TestSignalp(PluginTestBase):
  _plugin_name = "signalp4"

  def test_signalp4(self):
    if not self.params['signalp4_bin']:
      self.params['signalp4_bin'] = 'signalp'
    self.params['fasta'] = "input.fasta"
    self.params['signalp4_organism'] = 'gram+'
    signalp4.annotate(self.params, self.proteins)

    self.expected_output = {
        u'SPy_0252': True, 
        u'SPy_2077': False, 
        u'SPy_0317': True,
        u'sp|B7LNW7': True,
    }
    for seqid in self.expected_output:
      self.assertEqual(
          self.expected_output[seqid], self.proteins[seqid]['is_signalp'])
    self.assertEqual(self.proteins[u'sp|B7LNW7']['signalp_cleave_position'], 22)

if __name__ == '__main__':
  unittest.main()



import os
from inmembrane.helpers import dict_get, eval_surface_exposed_loop, \
                    chop_nterminal_peptide, log_stderr

    
def get_annotations(params):
  annotations = []
  params['signalp4_organism'] = 'gram-'
  
  if not params['signalp4_bin'] or params['signalp4_bin'] == 'signalp_web':
    annotations += ['signalp_web']
  else:
    annotations += ['signalp4']
  
  if not params['lipop1_bin'] or params['lipop1_bin'] == 'lipop_web':
    annotations += ['lipop_web']
  else:
    annotations += ['lipop1']
  
  annotations += ['tatfind_web']
  
  if 'bomp' in dict_get(params, 'barrel_programs'):
    annotations.append('bomp_web')
  if 'tmbhunt' in dict_get(params, 'barrel_programs'):
    annotations.append('tmbhunt_web')
  if 'tmbetadisc-rbf' in dict_get(params, 'barrel_programs'):
    annotations.append('tmbetadisc_rbf_web')
    
  # TMBETA-NET knows to only run on predicted barrels
  # with the category 'OM(barrel)'
  if 'tmbeta' in dict_get(params, 'barrel_programs'):
    annotations.append('tmbeta_net_web')

  if dict_get(params, 'helix_programs'):
    if 'tmhmm' in params['helix_programs']:
      if not params['tmhmm_bin'] or params['tmhmm_bin'] == 'tmhmm_web':
        annotations.append('tmhmm_web')
      else:
        annotations.append('tmhmm')
    if 'memsat3' in params['helix_programs']:
      annotations.append('memsat3')

  # run some hmm profiles to detect features (eg Tat signal)
  annotations += ['hmmsearch3']
  params['hmm_profiles_dir'] = os.path.join(
      os.path.dirname(__file__), 'gram_neg_profiles')

  return annotations

def post_process_protein(params, protein):
    
  def has_tm_helix(protein):
    for program in params['helix_programs']:
      if dict_get(protein, '%s_helices' % program):
        return True
    return False

  # these functions detect if and TM-containing IM proteins
  # have large loops / terminal regions in the periplasm or cytoplasm
  # that may be accessible / inaccessible in spheroplast shaving 
  # experiments.
  def has_long_loops(protein, loop_str='_outer_loops', \
                     loop_length=params['internal_exposed_loop_min']):
    for annot in protein:
      if loop_str in annot:
        for loop in protein[annot]:
          l_len = loop[1]-loop[0]
          if l_len >= loop_length:
            return True
    return False
  
  def long_in_periplasm(protein, \
                        loop_length=params['internal_exposed_loop_min']):
    return has_long_loops(protein, '_outer_loops', loop_length)
  
  def long_in_cytoplasm(protein, \
                        loop_length=params['internal_exposed_loop_min']):
    return has_long_loops(protein, '_inner_loops', loop_length)

  
  details = []
  category = "UNKNOWN"
  is_hmm_profile_match = dict_get(protein, 'hmmsearch')
  is_signalp = dict_get(protein, 'is_signalp')
  is_tatfind = dict_get(protein, 'is_tatfind')
  is_lipop = dict_get(protein, 'is_lipop')
  
  # in terms of most sublocalization logic, a Tat signal is similar to a 
  # Sec (signalp) signal. We use has_signal_pept to denote that either 
  # is present.
  has_signal_pept = False
  if is_signalp or is_tatfind or \
     (('hmmsearch' in protein) and "Tat_PS51318" in protein['hmmsearch']):
    has_signal_pept = True
  
  # annotate the barrels - high scoring bomp hits don't require a 
  # signal peptide, low scoring ones do
  has_barrel = False
  bomp_score = dict_get(protein, 'bomp')
  if (bomp_score >= params['bomp_clearly_cutoff']) or \
     (has_signal_pept and bomp_score >= params['bomp_maybe_cutoff']):
    
    details += ['bomp(%i)' % (bomp_score)]
    has_barrel = True
    
  tmbhunt_prob = dict_get(protein, 'tmbhunt_prob')
  if (tmbhunt_prob >= params['tmbhunt_clearly_cutoff']) or \
     (has_signal_pept and tmbhunt_prob >= params['tmbhunt_maybe_cutoff']):
    details += ['tmbhunt(%.2f)' % (tmbhunt_prob)]
    has_barrel = True
    
  if has_signal_pept and dict_get(protein, 'is_tmbetadisc_rbf'):
    details += ['tmbetadisc-rbf']
    has_barrel = True
    
  if has_barrel:
    category = 'OM(barrel)'
    
  # we only regard the barrel prediction as a true positive
  # if a signal peptide is also present
#  is_barrel = False
#  if has_signal_pept and has_barrel: # TODO and num_tms <= 1:
#    category = 'OM(barrel)'
#    is_barrel = True
    
  # set number of predicted OM barrel strands in details
  if has_barrel and \
      dict_get(protein, 'tmbeta_strands'):
    num_strands = len(protein['tmbeta_strands'])
    details += ['tmbeta_strands(%i)' % (num_strands)]
  
  if has_signal_pept and not is_lipop and \
    (dict_get(protein, 'signalp_cleave_position')):
    # we use the SignalP signal peptidase cleavage site for Tat signals
    chop_nterminal_peptide(protein,  protein['signalp_cleave_position'])
  
  if is_tatfind:
    details += ["tatfind"]
  
  if is_signalp:
    details += ["signalp"]
  
  if is_lipop:
    details += ["lipop"]
    chop_nterminal_peptide(protein, protein['lipop_cleave_position'])
  
  if is_hmm_profile_match:
    details += ["hmm(%s)" % "|".join(protein['hmmsearch'])]

  if has_tm_helix(protein) and not has_barrel:
    for program in params['helix_programs']:
      n = len(protein['%s_helices' % program])
      details += [program + "(%d)" % n]
    
    category = "IM"
    if long_in_periplasm(protein):
      category += "+peri"
    if long_in_cytoplasm(protein):
      category += "+cyto"
  elif not has_barrel:
    if is_lipop:
      if dict_get(protein, 'lipop_im_retention_signal'):
        category = "LIPOPROTEIN(IM)"
      else:
        category = "LIPOPROTEIN(OM)"
      pass
    elif (has_signal_pept):
      category = "PERIPLASMIC/SECRETED"
    else:
      category = "CYTOPLASM"

  if details == []:
    details = ["."]

  protein['details'] = details
  protein['category'] = category

  return details, category

def protein_output_line(seqid, proteins):
  return '%-15s   %-13s  %-50s  %s' % \
      (seqid, 
      proteins[seqid]['category'], 
      ";".join(proteins[seqid]['details']),
      proteins[seqid]['name'][:60])

def protein_csv_line(seqid, proteins):
  return '%s,%s,%s,"%s"\n' % \
      (seqid, 
       proteins[seqid]['category'], 
       ";".join(proteins[seqid]['details']),
       proteins[seqid]['name'])

def summary_table(params, proteins):
  """
  Returns a string representing a simple summary table of
  protein classifcations.
  """
  out = ""
  counts = {}
  for seqid in proteins:
    category = proteins[seqid]['category']
    
    if category not in counts:
      counts[category] = 1
    else:
      counts[category] += 1
      
  out += "\n\n# Number of proteins in each class:\n"
  for c in counts:
    out += "# %-15s %i\n" % (c, counts[c])
    
  return out



# -*- coding: utf-8 -*-

citation = {'ref': u"Gromiha MM, Ahmad S, Suwa M. Neural network-based "
                   u"prediction of transmembrane beta-strand segments in "
                   u"outer membrane proteins. Journal of computational "
                   u"chemistry 2004 Apr;25(5):762-7. \n"
                   u"<http://dx.doi.org/10.1002/jcc.10386>",
            'name': "TMBETA-NET"
           }

__DEBUG__ = False

import sys, os, time, StringIO
import json
import twill
from twill.commands import find, formfile, follow, fv, go, show, \
                             showforms, showlinks, submit, agent
                             
from inmembrane.helpers import dict_get, log_stderr

def annotate(params, proteins, \
                   url="http://psfs.cbrc.jp/tmbeta-net/", \
                   category='OM(barrel)',
                   force=False):
  """
  Uses the TMBETA-NET web service (http://psfs.cbrc.jp/tmbeta-net/) to
  predict strands of outer membrane beta-barrels.
  
  By default, category='BARREL' means prediction will only be run
  on proteins in the set with this category property. To process all
  proteins, change category to None.

  These keys are added to the proteins dictionary: 
    'tmbeta_strands' - a list of lists with paired start and end 
                       residues of each predicted strand. 
                       (eg [[3,9],[14,21], ..etc ])
  """

  # set the user-agent so web services can block us if they want ... :/
  python_version = sys.version.split()[0]
  agent("Python-urllib/%s (twill; inmembrane)" % python_version)
  
  outfile = 'tmbeta_net.out'
  log_stderr("# TMBETA-NET(web) %s > %s" % (params['fasta'], outfile))
  
  tmbeta_strands = {}
  if not force and os.path.isfile(outfile):
    log_stderr("# -> skipped: %s already exists" % outfile)
    fh = open(outfile, 'r')
    tmbeta_strands = json.loads(fh.read())
    fh.close()    
    for seqid in tmbeta_strands:
      proteins[seqid]['tmbeta_strands'] = tmbeta_strands[seqid]
      
    return tmbeta_strands

  # dump extraneous output into this blackhole so we don't see it
  if not __DEBUG__: twill.set_output(StringIO.StringIO())

  for seqid in proteins:
    
    # only run on sequences which match the category filter
    if force or \
       (category == None) or \
       (dict_get(proteins[seqid], 'category') == category):
      pass
    else:
      continue
      
    go(url)
    if __DEBUG__: showforms()
    fv("1","sequence",proteins[seqid]['seq'])
    submit()
    log_stderr("# TMBETA-NET: Predicting strands for %s - %s\n" \
                      % (seqid, proteins[seqid]['name']))
    out = show()
    time.sleep(1)
    
    # parse the web page returned, extract strand boundaries
    proteins[seqid]['tmbeta_strands'] = []
    for l in out.split('\n'):
      if __DEBUG__: log_stderr("## " + l)

      if "<BR>Segment " in l:
        i,j = l.split(":")[1].split("to")
        i = int(i.strip()[1:])
        j = int(j.strip()[1:])
        proteins[seqid]['tmbeta_strands'].append([i,j])

        if __DEBUG__: log_stderr("# TMBETA-NET(web) segments: %s, %s" % (i, j))

    tmbeta_strands[seqid] = proteins[seqid]['tmbeta_strands']

  # we store the parsed strand boundaries in JSON format
  fh = open(outfile, 'w')
  fh.write(json.dumps(tmbeta_strands, separators=(',',':\n')))
  fh.close()

  return tmbeta_strands



from distutils.core import setup

PACKAGE = "inmembrane"
DESCRIPTION = "A bioinformatic pipeline for proteome annotation \
to predict if a protein is exposed on the surface of a bacteria."
AUTHOR = "Andrew Perry & Bosco Ho"
AUTHOR_EMAIL = "ajperry@pansapiens.com"
URL = "http://github.com/boscoh/inmembrane"
VERSION = __import__(PACKAGE).__version__

setup(
    name=PACKAGE,
    version=VERSION,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    description=DESCRIPTION,
    packages=['inmembrane','inmembrane.plugins',
              'inmembrane.protocols','inmembrane.tests'],
    # NOTE: some packaging filters are also in MANIFEST.in
    package_data={'inmembrane': ['protocols/*/*', 
                                 'tests/*/*', 
                                 'plugins/*/*'], },
    scripts=['inmembrane_scan'],
    # README, examples & docs are included via MANIFEST.in
    license='BSD',
    long_description=open('README.rst', 'rt').read(),
    install_requires=[
        "suds >= 0.4",
        "BeautifulSoup >= 3.2.1",
        "twill >= 0.9",
    ],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Console",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: BSD License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
    ],
    zip_safe=False,
)



import os
import unittest
import sys

import inmembrane
import inmembrane.tests
from inmembrane import helpers
from inmembrane.plugins import tmbhunt_web

from inmembrane.tests.PluginTestBase import PluginTestBase

class TestTmbhunt(PluginTestBase):
  _plugin_name = "tmbhunt_web"
  
  def test_tmbhunt_web(self):
    self.expected_output = {
        'gi|107836852': {
            'tmbhunt_prob': 0.955956, 
            'tmbhunt': True
        }, 
        'gi|107837106': {
            'tmbhunt_prob': 0.23573599999999995, 
            'tmbhunt': False
        }, 
        'gi|107837101': {
            'tmbhunt_prob': 0.955956, 
            'tmbhunt': True
        }, 
        'gi|107836588': {
            'tmbhunt_prob': 0.903904, 
            'tmbhunt': True
        }, 
        'gi|107837107': {
            'tmbhunt_prob': 0.011001000000000039, 
            'tmbhunt': False
        }
    }

    # run TMB-HUNT
    self.output = tmbhunt_web.annotate(self.params, self.proteins, force=True)
    #print self.expected_output
    #helpers.print_proteins(self.output)
    for seqid in self.expected_output:
      self.assertIn(seqid, self.expected_output)
      if seqid in self.output:
        self.assertEqual(self.expected_output[seqid], self.output[seqid])

if __name__ == '__main__':
  unittest.main()



# -*- coding: utf-8 -*-

citation = {'ref': "Ou Y-YY, Gromiha MMM, Chen S-AA, Suwa M (2008) "
                   "TMBETADISC-RBF: Discrimination of beta-barrel "
                   "membrane proteins using RBF networks and PSSM profiles. "
                   "Computational biology and chemistry. \n"
                   "<http://dx.doi.org/10.1016/j.compbiolchem.2008.03.002>",
            'name': "TMBETADISC-RBF"
           }

__DEBUG__ = False

import sys, os, time, StringIO

import twill
from twill.commands import find, formfile, follow, fv, go, show, \
                             showforms, showlinks, submit, agent
from BeautifulSoup import BeautifulSoup                             
from inmembrane.helpers import log_stderr, parse_fasta_header, dict_get

def parse_tmbetadisc_output(output, proteins):
  """
  Parses the TMBETADISC-RBF output (file-like object or a list of strings) 
  an uses it to annotate and return an associated 'proteins' data structure.
  """

  soup = BeautifulSoup(output)
  # parse the table. we pop of single data cells one at a time
  fields = soup.findAll("td")
  fields.reverse()
  f = fields.pop() # discard first <td>1</td> field
  try:
    while len(fields) > 0:
      f = fields.pop().text
      seqid, result = parse_fasta_header(f)
      if "Non-Outer Membrane Protein" in result:
        proteins[seqid]["is_tmbetadisc_rbf"] = False
      elif "is Outer Membrane Protein" in result:
        proteins[seqid]["is_tmbetadisc_rbf"] = True
      fields.pop()
  except IndexError:
    # we get here when we run out of table fields to pop
    pass
        
  return proteins

def annotate(params, proteins, \
             url="http://rbf.bioinfo.tw/"+
                 "~sachen/OMPpredict/"+
                 "TMBETADISC-RBF-Content.html", force=False):
  """
  Interfaces with the TatFind web service at 
  (http://rbf.bioinfo.tw/~sachen/OMPpredict/TMBETADISC-RBF.php) 
  to predict if protein sequence is likely to be an outer membrane beta-barrel.
  
  Note that the default URL we use it different to the regular form used
  by web browsers, since we need to bypass some AJAX fun.
  """
  # TODO: automatically split large sets into multiple jobs
  #       since TMBETADISC seems to not like more than take 
  #       ~5000 seqs at a time
  if len(proteins) >= 5000:
    log_stderr("# ERROR: TMBETADISC-RBF(web): tends to fail with > ~5000 sequences.")
    return
  
  # set the user-agent so web services can block us if they want ... :/
  python_version = sys.version.split()[0]
  agent("Python-urllib/%s (twill; inmembrane)" % python_version)
  
  outfn = 'tmbetadisc-rbf.out'
  log_stderr("# TMBETADISC-RBF(web) %s > %s" % (params['fasta'], outfn))
  
  if not force and os.path.isfile(outfn):
    log_stderr("# -> skipped: %s already exists" % outfn)
    fh = open(outfn, 'r')
    proteins = parse_tmbetadisc_output(fh.read(), proteins)
    fh.close()
    return proteins
  
  # dump extraneous output into this blackhole so we don't see it
  if not __DEBUG__: twill.set_output(StringIO.StringIO())
  
  go(url)
  if __DEBUG__: showforms()
  formfile("1", "userfile", params["fasta"])
  fv("1", "format", "file")

  # set the user defined method
  method_map = {"aa":"Amino Acid Composition",
                "dp":"Depipetide Composition",
                "aadp":"Amino Acid & Depipetide Composition",
                "pssm":"PSSM"}
  if dict_get(params, 'tmbetadisc_rbf_method'):
    try:
      method = method_map[params['tmbetadisc_rbf_method']]
    except KeyError:
      log_stderr("# ERROR: Invalid setting from tmbetadisc_rbf_method. \
                    Must be set to aa, dp, aadp or pssm.")
      sys.exit()

  #fv("1", "select", "Amino Acid Composition")
  #fv("1", "select", "Depipetide Composition")
  #fv("1", "select", "Amino Acid & Depipetide Composition")
  #fv("1", "select", "PSSM")
  fv("1", "select", method)
  
  submit()
  
  waiting_page = show()
  if __DEBUG__: log_stderr(waiting_page)

  for l in waiting_page.split('\n'):
    if l.find("TMBETADISC-RBF-action.php?UniqueName=") != -1:
      result_url = l.split("'")[1]

  time.sleep(5)
  
  go(result_url)
  
  output = show()
  if __DEBUG__: log_stderr(output)
  
  # write raw output to a file
  fh = open(outfn, 'w')
  fh.write(output)
  fh.close()
  
  proteins = parse_tmbetadisc_output(output, proteins) 
  
  return proteins



# -*- coding: utf-8 -*-
__DEBUG__ = False

citation = {'ref': u"﻿Berven FS, Flikka K, Jensen HB, Eidhammer I (2004) "
                   u"BOMP: a program to predict integral beta-barrel outer "
                   u"membrane proteins encoded within genomes of Gram-negative "
                   u"bacteria. Nucleic acids research 32: W394-9. \n"
                   u"<http://dx.crossref.org/10.1093/nar/gkh351>",
            'name': "BOMP"
           }

import sys, os, time, StringIO

import twill
from twill.commands import find, formfile, follow, fv, go, show, \
                             showforms, showlinks, submit, agent
from BeautifulSoup import BeautifulSoup

from inmembrane.helpers import log_stderr, parse_fasta_header


def annotate(params, proteins, \
             url="http://services.cbu.uib.no/tools/bomp/", force=False):
  """
  Uses the BOMP web service (http://services.cbu.uib.no/tools/bomp/) to
  predict if proteins are outer membrane beta-barrels.
  """
  # set the user-agent so web services can block us if they want ... :/
  python_version = sys.version.split()[0]
  agent("Python-urllib/%s (twill; inmembrane)" % python_version)
  
  bomp_out = 'bomp.out'
  log_stderr("# BOMP(web) %s > %s" % (params['fasta'], bomp_out))
  
  if not force and os.path.isfile(bomp_out):
    log_stderr("# -> skipped: %s already exists" % bomp_out)
    bomp_categories = {}
    fh = open(bomp_out, 'r')
    for l in fh:
      words = l.split()
      bomp_category = int(words[-1:][0])
      seqid = parse_fasta_header(l)[0]
      proteins[seqid]['bomp'] = bomp_category
      bomp_categories[seqid] = bomp_category
    fh.close()
    return bomp_categories
  
  # dump extraneous output into this blackhole so we don't see it
  if not __DEBUG__: twill.set_output(StringIO.StringIO())
  
  go(url)
  if __DEBUG__: showforms()
  formfile("1", "queryfile", params["fasta"])
  submit()
  if __DEBUG__: show()
  
  # extract the job id from the page
  links = showlinks()
  job_id = None
  for l in links:
    if l.url.find("viewOutput") != -1:
      # grab job id from "viewOutput?id=16745338"
      job_id = int(l.url.split("=")[1])
  
  if __DEBUG__: log_stderr("BOMP job id: %d" % job_id)
  
  if not job_id:
    # something went wrong
    log_stderr("# BOMP error: Can't find job id")
    return
  
  # parse the HTML table and extract categories
  go("viewOutput?id=%i" % (job_id))
  
  polltime = 10
  log_stderr("# Waiting for BOMP to finish .")
  while True:
    try:
      find("Not finished")
      log_stderr(".")
    except:
      # Finished ! Pull down the result page.
      log_stderr(". done!\n")
      go("viewOutput?id=%i" % (job_id))
      if __DEBUG__: log_stderr(show())
      break
      
    # Not finished. We keep polling for a time until
    # we give up
    time.sleep(polltime)
    polltime = polltime * 2
    if polltime >= 7200: # 2 hours
      log_stderr("# BOMP error: Taking too long.")
      return
    go("viewOutput?id=%i" % (job_id))
    if __DEBUG__: log_stderr(show())
      
  bomp_html = show()
  if __DEBUG__: log_stderr(bomp_html)
  
  # Results are in the only <table> on this page, formatted like:
  # <tr><th>gi|107836852|gb|ABF84721.1<th>5</tr>
  soup = BeautifulSoup(bomp_html)
  bomp_categories = {} # dictionary of {name, category} pairs
  for tr in soup.findAll('tr')[1:]:
    n, c = tr.findAll('th')
    name = parse_fasta_header(n.text.strip())[0]
    category = int(c.text)
    bomp_categories[name] = category
  
  # write BOMP results to a tab delimited file
  fh = open(bomp_out, 'w')
  for k,v in bomp_categories.iteritems():
    fh.write("%s\t%i\n" % (k,v))
  fh.close()
  
  if __DEBUG__: log_stderr(str(bomp_categories))
  
  # label proteins with bomp classification (int) or False
  for name in proteins:
    if "bomp" not in proteins[name]:
      if name in bomp_categories:
        category = int(bomp_categories[name])
        proteins[name]['bomp'] = category
      else:
        proteins[name]['bomp'] = False
  
  if __DEBUG__: log_stderr(str(proteins))
  
  return bomp_categories
  
  """
  # Alternative: just get binary classification results via the
  #              FASTA output BOMP links to
  #
  # use the job id to jump straight to the fasta results
  # if a sequence is here, it's classified as an OMP barrel
  go("viewFasta?id=%i" % (job_id))
  bomp_seqs = show()
  bomp_fasta_headers = read_fasta_keys(StringIO.StringIO(show()))
  # label the predicted TMBs
  for name in bomp_fasta_headers:
    proteins[name]['bomp'] = True
    
  # label all the non-TMBs
  for name in proteins:
    if "bomp" not in proteins[name]:
      proteins[name]['bomp'] = False
  """



import os
from inmembrane.helpers import dict_get, eval_surface_exposed_loop, \
                    chop_nterminal_peptide

def get_annotations(params):
  """
  Creates a list of annotation functions required
  by this gram_pos protocol. The main program
  will run the annotation functions of this list,
  mapping the correct functions to the strings.

  As well, the function does some bookeeping on
  params to make sure the 'hmm_profiles_dir' is
  pointing in the right place.
  """
  annotations = []
  
  params['signalp4_organism'] = 'gram+'
  
  if not params['signalp4_bin'] or params['signalp4_bin'] == 'signalp_web':
    annotations += ['signalp_web']
  else:
    annotations += ['signalp4']
  
  if not params['lipop1_bin'] or params['lipop1_bin'] == 'lipop_web':
    annotations += ['lipop_web']
  else:
    annotations += ['lipop1']
    
  annotations += ['hmmsearch3']

  if dict_get(params, 'helix_programs'):
    if 'tmhmm' in params['helix_programs']:
      if not params['tmhmm_bin'] or params['tmhmm_bin'] == 'tmhmm_web':
        annotations.append('tmhmm_web')
      else:
        annotations.append('tmhmm')
    if 'memsat3' in params['helix_programs']:
      annotations.append('memsat3')

  params['hmm_profiles_dir'] = os.path.join(
      os.path.dirname(__file__), 'gram_pos_profiles')

  return annotations


def post_process_protein(params, protein):
  """
  This is the main analysis of the protein, where theprotein
  dictionary should contain all the necessary information
  from the annotations. Thus post_process_protein contain
  can determine the final analysis.
  """

  def sequence_length(protein):
    return protein['sequence_length']
    
  def has_tm_helix(protein):
    for program in params['helix_programs']:
      if dict_get(protein, '%s_helices' % program):
        return True
    return False

  def has_surface_exposed_loop(protein):
    for program in params['helix_programs']:
      if eval_surface_exposed_loop(
          protein['sequence_length'], 
          len(protein['%s_helices' % (program)]), 
          protein['%s_outer_loops' % (program)], 
          params['terminal_exposed_loop_min'], 
          params['internal_exposed_loop_min']):
        return True
    return False

  terminal_exposed_loop_min = \
      params['terminal_exposed_loop_min']

  is_hmm_profile_match = dict_get(protein, 'hmmsearch')
  is_lipop = dict_get(protein, 'is_lipop')
  if is_lipop:
    i_lipop_cut = protein['lipop_cleave_position']
  is_signalp = dict_get(protein, 'is_signalp')
  if is_signalp:
    i_signalp_cut = protein['signalp_cleave_position']

  details = []
  if is_hmm_profile_match:
    details += ["hmm(%s)" % "|".join(protein['hmmsearch'])]
  if is_lipop: 
    details += ["lipop"]
  if is_signalp:
    details += ["signalp"]
  for program in params['helix_programs']:
    if has_tm_helix(protein):
      n = len(protein['%s_helices' % program])
      details += [program + "(%d)" % n]

  if is_lipop:
    chop_nterminal_peptide(protein, i_lipop_cut)
  elif is_signalp:
    chop_nterminal_peptide(protein, i_signalp_cut)

  if is_hmm_profile_match:
    category =  "PSE-Cellwall"
  elif has_tm_helix(protein):
    if has_surface_exposed_loop(protein):
      category = "PSE-Membrane"
    else:
      category = "MEMBRANE(non-PSE)"
  else:
    if is_lipop:
      # whole protein considered outer terminal loop
      if sequence_length(protein) < terminal_exposed_loop_min:
        category = "LIPOPROTEIN(non-PSE)"
      else:
        category = "PSE-Lipoprotein"
    elif is_signalp:
      category = "SECRETED"
    else:
      category = "CYTOPLASM(non-PSE)"

  if details == []:
    details = ["."]

  protein['details'] = details
  protein['category'] = category

  return details, category


def protein_output_line(seqid, proteins):
  return '%-15s   %-18s  %-50s  %s' % \
      (seqid, 
      proteins[seqid]['category'], 
      ";".join(proteins[seqid]['details']),
      proteins[seqid]['name'][:60])

def protein_csv_line(seqid, proteins):
  return '%s,%s,%s,"%s"\n' % \
      (seqid, 
       proteins[seqid]['category'], 
       ";".join(proteins[seqid]['details']),
       proteins[seqid]['name'])


def summary_table(params, proteins):
  """
  Returns a string representing a simple summary table of
  protein classifcations.
  """
  out = ""
  counts = {}
  for seqid in proteins:
    category = proteins[seqid]['category']
    
    if category not in counts:
      counts[category] = 1
    else:
      counts[category] += 1
      
  out += "\n\n# Number of proteins in each class:\n"
  pse_total = 0
  
  for c in counts:
    if "PSE-" in c:
      pse_total += counts[c]
  counts["PSE(total)"] = pse_total
  
  for c in sorted(counts):
    if "PSE-" in c:
      spacer = "  "
      pse_total += counts[c]
    else:
      spacer = ""
    out += "# %s%-15s\t%i\n" % (spacer, c, counts[c])
  
  #out += "#\n# PSE(total)\t\t%i\n" % (pse_total)

  return out
